# encoding: utf-8

"""Test suite for pptx.text.text module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.compat import is_unicode
from pptx.dml.color import ColorFormat
from pptx.dml.fill import FillFormat
from pptx.enum.lang import MSO_LANGUAGE_ID
from pptx.enum.text import MSO_ANCHOR, MSO_AUTO_SIZE, MSO_UNDERLINE, PP_ALIGN
from pptx.opc.constants import RELATIONSHIP_TYPE as RT
from pptx.opc.package import Part
from pptx.shapes.autoshape import Shape
from pptx.text.text import Font, _Hyperlink, _Paragraph, _Run, TextFrame
from pptx.util import Inches, Pt

from ..oxml.unitdata.text import a_p, a_t, an_hlinkClick, an_r, an_rPr
from ..unitutil.cxml import element, xml
from ..unitutil.mock import (
    class_mock,
    instance_mock,
    loose_mock,
    method_mock,
    property_mock,
)


class DescribeTextFrame(object):
    """Unit-test suite for `pptx.text.text.TextFrame` object."""

    def it_can_add_a_paragraph_to_itself(self, add_paragraph_fixture):
        text_frame, expected_xml = add_paragraph_fixture
        text_frame.add_paragraph()
        assert text_frame._txBody.xml == expected_xml

    def it_knows_its_autosize_setting(self, autosize_get_fixture):
        text_frame, expected_value = autosize_get_fixture
        assert text_frame.auto_size == expected_value

    def it_can_change_its_autosize_setting(self, autosize_set_fixture):
        text_frame, value, expected_xml = autosize_set_fixture
        text_frame.auto_size = value
        assert text_frame._txBody.xml == expected_xml

    def it_knows_its_margin_settings(self, margin_get_fixture):
        text_frame, prop_name, unit, expected_value = margin_get_fixture
        margin_value = getattr(text_frame, prop_name)
        assert getattr(margin_value, unit) == expected_value

    def it_can_change_its_margin_settings(self, margin_set_fixture):
        text_frame, prop_name, new_value, expected_xml = margin_set_fixture
        setattr(text_frame, prop_name, new_value)
        assert text_frame._txBody.xml == expected_xml

    def it_knows_its_vertical_alignment(self, anchor_get_fixture):
        text_frame, expected_value = anchor_get_fixture
        assert text_frame.vertical_anchor == expected_value

    def it_can_change_its_vertical_alignment(self, anchor_set_fixture):
        text_frame, new_value, expected_xml = anchor_set_fixture
        text_frame.vertical_anchor = new_value
        assert text_frame._element.xml == expected_xml

    def it_knows_its_word_wrap_setting(self, wrap_get_fixture):
        text_frame, expected_value = wrap_get_fixture
        assert text_frame.word_wrap == expected_value

    def it_can_change_its_word_wrap_setting(self, wrap_set_fixture):
        text_frame, new_value, expected_xml = wrap_set_fixture
        text_frame.word_wrap = new_value
        assert text_frame._element.xml == expected_xml

    def it_provides_access_to_its_paragraphs(self, paragraphs_fixture):
        text_frame, ps = paragraphs_fixture
        paragraphs = text_frame.paragraphs
        assert len(paragraphs) == len(ps)
        for idx, paragraph in enumerate(paragraphs):
            assert isinstance(paragraph, _Paragraph)
            assert paragraph._element is ps[idx]

    def it_raises_on_attempt_to_set_margin_to_non_int(self):
        text_frame = TextFrame(element("p:txBody/a:bodyPr"), None)
        with pytest.raises(TypeError):
            text_frame.margin_bottom = "0.1"

    def it_knows_the_part_it_belongs_to(self, text_frame_with_parent_):
        text_frame, parent_ = text_frame_with_parent_
        part = text_frame.part
        assert part is parent_.part

    def it_knows_what_text_it_contains(
        self, request, text_get_fixture, paragraphs_prop_
    ):
        paragraph_texts, expected_value = text_get_fixture
        paragraphs_prop_.return_value = tuple(
            instance_mock(request, _Paragraph, text=text) for text in paragraph_texts
        )
        text_frame = TextFrame(None, None)

        text = text_frame.text

        assert text == expected_value

    def it_can_replace_the_text_it_contains(self, text_set_fixture):
        txBody, text, expected_xml = text_set_fixture
        text_frame = TextFrame(txBody, None)

        text_frame.text = text

        assert text_frame._element.xml == expected_xml

    def it_can_resize_its_text_to_best_fit(
        self, text_prop_, _best_fit_font_size_, _apply_fit_
    ):
        family, max_size, bold, italic, font_file, font_size = (
            "Family",
            42,
            "bold",
            "italic",
            "font_file",
            21,
        )
        text_prop_.return_value = "some text"
        _best_fit_font_size_.return_value = font_size
        text_frame = TextFrame(None, None)

        text_frame.fit_text(family, max_size, bold, italic, font_file)

        text_frame._best_fit_font_size.assert_called_once_with(
            family, max_size, bold, italic, font_file
        )
        text_frame._apply_fit.assert_called_once_with(family, font_size, bold, italic)

    def it_calculates_its_best_fit_font_size_to_help_fit_text(self, size_font_fixture):
        text_frame, family, max_size, bold, italic = size_font_fixture[:5]
        FontFiles_, TextFitter_, text, extents = size_font_fixture[5:9]
        font_file_, font_size_ = size_font_fixture[9:]

        font_size = text_frame._best_fit_font_size(family, max_size, bold, italic, None)

        FontFiles_.find.assert_called_once_with(family, bold, italic)
        TextFitter_.best_fit_font_size.assert_called_once_with(
            text, extents, max_size, font_file_
        )
        assert font_size is font_size_

    def it_calculates_its_effective_size_to_help_fit_text(self):
        sp_cxml = (
            "p:sp/(p:spPr/a:xfrm/(a:off{x=914400,y=914400},a:ext{cx=914400,c"
            "y=914400}),p:txBody/(a:bodyPr,a:p))"
        )
        text_frame = Shape(element(sp_cxml), None).text_frame
        assert text_frame._extents == (731520, 822960)

    def it_applies_fit_to_help_fit_text(self, apply_fit_fixture):
        text_frame, family, font_size, bold, italic = apply_fit_fixture
        text_frame._apply_fit(family, font_size, bold, italic)
        assert text_frame.auto_size is MSO_AUTO_SIZE.NONE
        assert text_frame.word_wrap is True
        text_frame._set_font.assert_called_once_with(family, font_size, bold, italic)

    def it_sets_its_font_to_help_fit_text(self, set_font_fixture):
        text_frame, family, size, bold, italic, expected_xml = set_font_fixture
        text_frame._set_font(family, size, bold, italic)
        assert text_frame._element.xml == expected_xml

    # fixtures ---------------------------------------------

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", "p:txBody/(a:bodyPr,a:p)"),
            ("p:txBody/(a:bodyPr,a:p)", "p:txBody/(a:bodyPr,a:p,a:p)"),
        ]
    )
    def add_paragraph_fixture(self, request):
        txBody_cxml, expected_cxml = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        expected_xml = xml(expected_cxml)
        return text_frame, expected_xml

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", None),
            ("p:txBody/a:bodyPr{anchor=t}", MSO_ANCHOR.TOP),
            ("p:txBody/a:bodyPr{anchor=b}", MSO_ANCHOR.BOTTOM),
        ]
    )
    def anchor_get_fixture(self, request):
        txBody_cxml, expected_value = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        return text_frame, expected_value

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", MSO_ANCHOR.TOP, "p:txBody/a:bodyPr{anchor=t}"),
            (
                "p:txBody/a:bodyPr{anchor=t}",
                MSO_ANCHOR.MIDDLE,
                "p:txBody/a:bodyPr{anchor=ctr}",
            ),
            (
                "p:txBody/a:bodyPr{anchor=ctr}",
                MSO_ANCHOR.BOTTOM,
                "p:txBody/a:bodyPr{anchor=b}",
            ),
            ("p:txBody/a:bodyPr{anchor=b}", None, "p:txBody/a:bodyPr"),
        ]
    )
    def anchor_set_fixture(self, request):
        txBody_cxml, new_value, expected_cxml = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        expected_xml = xml(expected_cxml)
        return text_frame, new_value, expected_xml

    @pytest.fixture
    def apply_fit_fixture(self, _set_font_):
        txBody = element("p:txBody/a:bodyPr")
        text_frame = TextFrame(txBody, None)
        family, font_size, bold, italic = "Family", 42, True, False
        return text_frame, family, font_size, bold, italic

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", None),
            ("p:txBody/a:bodyPr/a:noAutofit", MSO_AUTO_SIZE.NONE),
            ("p:txBody/a:bodyPr/a:spAutoFit", MSO_AUTO_SIZE.SHAPE_TO_FIT_TEXT),
            ("p:txBody/a:bodyPr/a:normAutofit", MSO_AUTO_SIZE.TEXT_TO_FIT_SHAPE),
        ]
    )
    def autosize_get_fixture(self, request):
        txBody_cxml, expected_value = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        return text_frame, expected_value

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", MSO_AUTO_SIZE.NONE, "p:txBody/a:bodyPr/a:noAutofit"),
            (
                "p:txBody/a:bodyPr/a:noAutofit",
                MSO_AUTO_SIZE.SHAPE_TO_FIT_TEXT,
                "p:txBody/a:bodyPr/a:spAutoFit",
            ),
            (
                "p:txBody/a:bodyPr/a:spAutoFit",
                MSO_AUTO_SIZE.TEXT_TO_FIT_SHAPE,
                "p:txBody/a:bodyPr/a:normAutofit",
            ),
            ("p:txBody/a:bodyPr/a:normAutofit", None, "p:txBody/a:bodyPr"),
        ]
    )
    def autosize_set_fixture(self, request):
        txBody_cxml, value, expected_cxml = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        expected_xml = xml(expected_cxml)
        return text_frame, value, expected_xml

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", "left", "emu", Inches(0.1)),
            ("p:txBody/a:bodyPr", "top", "emu", Inches(0.05)),
            ("p:txBody/a:bodyPr", "right", "emu", Inches(0.1)),
            ("p:txBody/a:bodyPr", "bottom", "emu", Inches(0.05)),
            ("p:txBody/a:bodyPr{lIns=9144}", "left", "cm", 0.0254),
            ("p:txBody/a:bodyPr{tIns=18288}", "top", "mm", 0.508),
            ("p:txBody/a:bodyPr{rIns=76200}", "right", "pt", 6.0),
            ("p:txBody/a:bodyPr{bIns=36576}", "bottom", "inches", 0.04),
        ]
    )
    def margin_get_fixture(self, request):
        txBody_cxml, side, unit, expected_value = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        prop_name = "margin_%s" % side
        return text_frame, prop_name, unit, expected_value

    @pytest.fixture(
        params=[
            (
                "p:txBody/a:bodyPr",
                "left",
                Inches(0.11),
                "p:txBody/a:bodyPr{lIns=100584}",
            ),
            (
                "p:txBody/a:bodyPr{tIns=1234}",
                "top",
                Inches(0.12),
                "p:txBody/a:bodyPr{tIns=109728}",
            ),
            (
                "p:txBody/a:bodyPr{rIns=2345}",
                "right",
                Inches(0.13),
                "p:txBody/a:bodyPr{rIns=118872}",
            ),
            (
                "p:txBody/a:bodyPr{bIns=3456}",
                "bottom",
                Inches(0.14),
                "p:txBody/a:bodyPr{bIns=128016}",
            ),
            ("p:txBody/a:bodyPr", "left", Inches(0.1), "p:txBody/a:bodyPr"),
            ("p:txBody/a:bodyPr", "top", Inches(0.05), "p:txBody/a:bodyPr"),
            ("p:txBody/a:bodyPr", "right", Inches(0.1), "p:txBody/a:bodyPr"),
            ("p:txBody/a:bodyPr", "bottom", Inches(0.05), "p:txBody/a:bodyPr"),
        ]
    )
    def margin_set_fixture(self, request):
        txBody_cxml, side, new_value, expected_txBody_cxml = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        prop_name = "margin_%s" % side
        expected_xml = xml(expected_txBody_cxml)
        return text_frame, prop_name, new_value, expected_xml

    @pytest.fixture(params=["p:txBody", "p:txBody/a:p", "p:txBody/(a:p,a:p)"])
    def paragraphs_fixture(self, request):
        txBody_cxml = request.param
        txBody = element(txBody_cxml)
        text_frame = TextFrame(txBody, None)
        ps = txBody.xpath(".//a:p")
        return text_frame, ps

    @pytest.fixture(
        params=[
            (
                "p:txBody/(a:bodyPr,a:p/a:r)",
                True,
                False,
                "p:txBody/(a:bodyPr,a:p/(a:r/a:rPr{sz=600,b=1,i=0}/a:latin{typeface"
                "=F},a:endParaRPr{sz=600,b=1,i=0}/a:latin{typeface=F}))",
            ),
            (
                "p:txBody/a:p/a:br",
                True,
                False,
                "p:txBody/a:p/(a:br/a:rPr{sz=600,b=1,i=0}/a:latin{typeface=F},a:end"
                "ParaRPr{sz=600,b=1,i=0}/a:latin{typeface=F})",
            ),
            (
                "p:txBody/a:p/a:fld",
                True,
                False,
                "p:txBody/a:p/(a:fld/a:rPr{sz=600,b=1,i=0}/a:latin{typeface=F},a:en"
                "dParaRPr{sz=600,b=1,i=0}/a:latin{typeface=F})",
            ),
        ]
    )
    def set_font_fixture(self, request):
        txBody_cxml, bold, italic, expected_cxml = request.param
        family, size = "F", 6
        text_frame = TextFrame(element(txBody_cxml), None)
        expected_xml = xml(expected_cxml)
        return text_frame, family, size, bold, italic, expected_xml

    @pytest.fixture
    def size_font_fixture(self, FontFiles_, TextFitter_, text_prop_, _extents_prop_):
        text_frame = TextFrame(None, None)
        family, max_size, bold, italic = "Family", 42, True, False
        text, extents, font_size, font_file = "text", (111, 222), 21, "f.ttf"
        text_prop_.return_value = text
        _extents_prop_.return_value = extents
        FontFiles_.find.return_value = font_file
        TextFitter_.best_fit_font_size.return_value = font_size
        return (
            text_frame,
            family,
            max_size,
            bold,
            italic,
            FontFiles_,
            TextFitter_,
            text,
            extents,
            font_file,
            font_size,
        )

    @pytest.fixture(
        params=[(["foobar"], "foobar"), (["foo", "bar", "baz"], "foo\nbar\nbaz")]
    )
    def text_get_fixture(self, request):
        paragraph_texts, expected_value = request.param
        return paragraph_texts, expected_value

    @pytest.fixture(
        params=[
            # ---empty to something---
            ("p:txBody/a:p", "foobar", 'p:txBody/a:p/a:r/a:t"foobar"'),
            # ---something to something else---
            ('p:txBody/a:p/a:r/a:t"foobar"', "barfoo", 'p:txBody/a:p/a:r/a:t"barfoo"'),
            # ---single paragraph to multiple---
            (
                'p:txBody/a:p/a:r/a:t"barfoo"',
                "foo\nbar",
                'p:txBody/(a:p/a:r/a:t"foo",a:p/a:r/a:t"bar")',
            ),
            # ---multiple paragraphs to single---
            (
                'p:txBody/(a:p/a:r/a:t"foo",a:p/a:r/a:t"bar")',
                "barfoo",
                'p:txBody/a:p/a:r/a:t"barfoo"',
            ),
            # ---something to empty---
            ('p:txBody/a:p/a:r/a:t"foobar"', "", "p:txBody/a:p"),
            # ---vertical-tab becomes line-break---
            ("p:txBody/a:p", "a\vb", 'p:txBody/a:p/(a:r/a:t"a",a:br,a:r/a:t"b")'),
        ]
    )
    def text_set_fixture(self, request):
        txBody_cxml, text, expected_cxml = request.param
        txBody = element(txBody_cxml)
        expected_xml = xml(expected_cxml)
        return txBody, text, expected_xml

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", None),
            ("p:txBody/a:bodyPr{wrap=square}", True),
            ("p:txBody/a:bodyPr{wrap=none}", False),
        ]
    )
    def wrap_get_fixture(self, request):
        txBody_cxml, expected_value = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        return text_frame, expected_value

    @pytest.fixture(
        params=[
            ("p:txBody/a:bodyPr", True, "p:txBody/a:bodyPr{wrap=square}"),
            ("p:txBody/a:bodyPr{wrap=square}", False, "p:txBody/a:bodyPr{wrap=none}"),
            ("p:txBody/a:bodyPr{wrap=none}", None, "p:txBody/a:bodyPr"),
        ]
    )
    def wrap_set_fixture(self, request):
        txBody_cxml, new_value, expected_txBody_cxml = request.param
        text_frame = TextFrame(element(txBody_cxml), None)
        expected_xml = xml(expected_txBody_cxml)
        return text_frame, new_value, expected_xml

    # fixture components -----------------------------------

    @pytest.fixture
    def _apply_fit_(self, request):
        return method_mock(request, TextFrame, "_apply_fit")

    @pytest.fixture
    def _best_fit_font_size_(self, request):
        return method_mock(request, TextFrame, "_best_fit_font_size")

    @pytest.fixture
    def _extents_prop_(self, request):
        return property_mock(request, TextFrame, "_extents")

    @pytest.fixture
    def FontFiles_(self, request):
        return class_mock(request, "pptx.text.text.FontFiles")

    @pytest.fixture
    def paragraphs_prop_(self, request):
        return property_mock(request, TextFrame, "paragraphs")

    @pytest.fixture
    def _set_font_(self, request):
        return method_mock(request, TextFrame, "_set_font")

    @pytest.fixture
    def TextFitter_(self, request):
        return class_mock(request, "pptx.text.text.TextFitter")

    @pytest.fixture
    def text_frame_with_parent_(self, request):
        parent_ = loose_mock(request, name="parent_")
        text_frame = TextFrame(None, parent_)
        return text_frame, parent_

    @pytest.fixture
    def text_prop_(self, request):
        return property_mock(request, TextFrame, "text")


class DescribeFont(object):
    def it_knows_its_bold_setting(self, bold_get_fixture):
        font, expected_value = bold_get_fixture
        assert font.bold == expected_value

    def it_can_change_its_bold_setting(self, bold_set_fixture):
        font, new_value, expected_xml = bold_set_fixture
        font.bold = new_value
        assert font._element.xml == expected_xml

    def it_knows_its_italic_setting(self, italic_get_fixture):
        font, expected_value = italic_get_fixture
        assert font.italic == expected_value

    def it_can_change_its_italic_setting(self, italic_set_fixture):
        font, new_value, expected_xml = italic_set_fixture
        font.italic = new_value
        assert font._element.xml == expected_xml

    def it_knows_its_language_id(self, language_id_get_fixture):
        font, expected_value = language_id_get_fixture
        assert font.language_id == expected_value

    def it_can_change_its_language_id_setting(self, language_id_set_fixture):
        font, new_value, expected_xml = language_id_set_fixture
        font.language_id = new_value
        assert font._element.xml == expected_xml

    def it_knows_its_underline_setting(self, underline_get_fixture):
        font, expected_value = underline_get_fixture
        assert font.underline is expected_value, "got %s" % font.underline

    def it_can_change_its_underline_setting(self, underline_set_fixture):
        font, new_value, expected_xml = underline_set_fixture
        font.underline = new_value
        assert font._element.xml == expected_xml

    def it_knows_its_size(self, size_get_fixture):
        font, expected_value = size_get_fixture
        assert font.size == expected_value

    def it_can_change_its_size(self, size_set_fixture):
        font, new_value, expected_xml = size_set_fixture
        font.size = new_value
        assert font._element.xml == expected_xml

    def it_knows_its_latin_typeface(self, name_get_fixture):
        font, expected_value = name_get_fixture
        assert font.name == expected_value

    def it_can_change_its_latin_typeface(self, name_set_fixture):
        font, new_value, expected_xml = name_set_fixture
        font.name = new_value
        assert font._element.xml == expected_xml

    def it_provides_access_to_its_color(self, font):
        assert isinstance(font.color, ColorFormat)

    def it_provides_access_to_its_fill(self, font):
        assert isinstance(font.fill, FillFormat)

    # fixtures ---------------------------------------------

    @pytest.fixture(
        params=[("a:rPr", None), ("a:rPr{b=0}", False), ("a:rPr{b=1}", True)]
    )
    def bold_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[
            ("a:rPr", True, "a:rPr{b=1}"),
            ("a:rPr{b=1}", False, "a:rPr{b=0}"),
            ("a:rPr{b=0}", None, "a:rPr"),
        ]
    )
    def bold_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    @pytest.fixture(
        params=[("a:rPr", None), ("a:rPr{i=0}", False), ("a:rPr{i=1}", True)]
    )
    def italic_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[
            ("a:rPr", True, "a:rPr{i=1}"),
            ("a:rPr{i=1}", False, "a:rPr{i=0}"),
            ("a:rPr{i=0}", None, "a:rPr"),
        ]
    )
    def italic_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("a:rPr", MSO_LANGUAGE_ID.NONE),
            ("a:rPr{lang=pl-PL}", MSO_LANGUAGE_ID.POLISH),
            ("a:rPr{lang=de-AT}", MSO_LANGUAGE_ID.GERMAN_AUSTRIA),
            ("a:rPr{lang=fr-FR}", MSO_LANGUAGE_ID.FRENCH),
        ]
    )
    def language_id_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[
            ("a:rPr", MSO_LANGUAGE_ID.ZULU, "a:rPr{lang=zu-ZA}"),
            ("a:rPr{lang=zu-ZA}", MSO_LANGUAGE_ID.URDU, "a:rPr{lang=ur-PK}"),
            ("a:rPr{lang=ur-PK}", MSO_LANGUAGE_ID.NONE, "a:rPr"),
            ("a:rPr{lang=ur-PK}", None, "a:rPr"),
            ("a:rPr", MSO_LANGUAGE_ID.NONE, "a:rPr"),
            ("a:rPr", None, "a:rPr"),
        ]
    )
    def language_id_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    @pytest.fixture(
        params=[("a:rPr", None), ("a:rPr/a:latin{typeface=Foobar}", "Foobar")]
    )
    def name_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[
            ("a:rPr", "Foobar", "a:rPr/a:latin{typeface=Foobar}"),
            (
                "a:rPr/a:latin{typeface=Foobar}",
                "Barfoo",
                "a:rPr/a:latin{typeface=Barfoo}",
            ),
            ("a:rPr/a:latin{typeface=Barfoo}", None, "a:rPr"),
        ]
    )
    def name_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    @pytest.fixture(params=[("a:rPr", None), ("a:rPr{sz=2400}", 304800)])
    def size_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[("a:rPr", Pt(24), "a:rPr{sz=2400}"), ("a:rPr{sz=2400}", None, "a:rPr")]
    )
    def size_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("a:rPr", None),
            ("a:rPr{u=none}", False),
            ("a:rPr{u=sng}", True),
            ("a:rPr{u=dbl}", MSO_UNDERLINE.DOUBLE_LINE),
        ]
    )
    def underline_get_fixture(self, request):
        rPr_cxml, expected_value = request.param
        font = Font(element(rPr_cxml))
        return font, expected_value

    @pytest.fixture(
        params=[
            ("a:rPr", True, "a:rPr{u=sng}"),
            ("a:rPr{u=sng}", False, "a:rPr{u=none}"),
            ("a:rPr{u=none}", MSO_UNDERLINE.WAVY_LINE, "a:rPr{u=wavy}"),
            ("a:rPr{u=wavy}", MSO_UNDERLINE.NONE, "a:rPr{u=none}"),
            ("a:rPr{u=wavy}", None, "a:rPr"),
        ]
    )
    def underline_set_fixture(self, request):
        rPr_cxml, new_value, expected_rPr_cxml = request.param
        font = Font(element(rPr_cxml))
        expected_xml = xml(expected_rPr_cxml)
        return font, new_value, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def font(self):
        return Font(element("a:rPr"))


class Describe_Hyperlink(object):
    def it_knows_the_target_url_of_the_hyperlink(self, hlink_with_url_):
        hlink, rId, url = hlink_with_url_
        assert hlink.address == url
        hlink.part.target_ref.assert_called_once_with(rId)

    def it_has_None_for_address_when_no_hyperlink_is_present(self, hlink):
        assert hlink.address is None

    def it_can_set_the_target_url(self, hlink, rPr_with_hlinkClick_xml, url):
        hlink.address = url
        # verify -----------------------
        hlink.part.relate_to.assert_called_once_with(
            url, RT.HYPERLINK, is_external=True
        )
        assert hlink._rPr.xml == rPr_with_hlinkClick_xml
        assert hlink.address == url

    def it_can_remove_the_hyperlink(self, remove_hlink_fixture_):
        hlink, rPr_xml, rId = remove_hlink_fixture_
        hlink.address = None
        assert hlink._rPr.xml == rPr_xml
        hlink.part.drop_rel.assert_called_once_with(rId)

    def it_should_remove_the_hyperlink_when_url_set_to_empty_string(
        self, remove_hlink_fixture_
    ):
        hlink, rPr_xml, rId = remove_hlink_fixture_
        hlink.address = ""
        assert hlink._rPr.xml == rPr_xml
        hlink.part.drop_rel.assert_called_once_with(rId)

    def it_can_change_the_target_url(self, change_hlink_fixture_):
        # fixture ----------------------
        hlink, rId_existing, new_url, new_rPr_xml = change_hlink_fixture_
        # exercise ---------------------
        hlink.address = new_url
        # verify -----------------------
        assert hlink._rPr.xml == new_rPr_xml
        hlink.part.drop_rel.assert_called_once_with(rId_existing)
        hlink.part.relate_to.assert_called_once_with(
            new_url, RT.HYPERLINK, is_external=True
        )

    # fixtures ---------------------------------------------

    @pytest.fixture
    def change_hlink_fixture_(
        self, request, hlink_with_hlinkClick, rId, rId_2, part_, url_2
    ):
        hlinkClick_bldr = an_hlinkClick().with_rId(rId_2)
        new_rPr_xml = an_rPr().with_nsdecls("a", "r").with_child(hlinkClick_bldr).xml()
        part_.relate_to.return_value = rId_2
        property_mock(request, _Hyperlink, "part", return_value=part_)
        return hlink_with_hlinkClick, rId, url_2, new_rPr_xml

    @pytest.fixture
    def hlink(self, request, part_):
        rPr = an_rPr().with_nsdecls("a", "r").element
        hlink = _Hyperlink(rPr, None)
        property_mock(request, _Hyperlink, "part", return_value=part_)
        return hlink

    @pytest.fixture
    def hlink_with_hlinkClick(self, request, rPr_with_hlinkClick_bldr):
        rPr = rPr_with_hlinkClick_bldr.element
        return _Hyperlink(rPr, None)

    @pytest.fixture
    def hlink_with_url_(self, request, part_, hlink_with_hlinkClick, rId, url):
        property_mock(request, _Hyperlink, "part", return_value=part_)
        return hlink_with_hlinkClick, rId, url

    @pytest.fixture
    def part_(self, request, url, rId):
        """
        Mock Part instance suitable for patching into _Hyperlink.part
        property. It returns url for target_ref() and rId for relate_to().
        """
        part_ = instance_mock(request, Part)
        part_.target_ref.return_value = url
        part_.relate_to.return_value = rId
        return part_

    @pytest.fixture
    def rId(self):
        return "rId2"

    @pytest.fixture
    def rId_2(self):
        return "rId6"

    @pytest.fixture
    def remove_hlink_fixture_(self, request, hlink_with_hlinkClick, rPr_xml, rId):
        property_mock(request, _Hyperlink, "part")
        return hlink_with_hlinkClick, rPr_xml, rId

    @pytest.fixture
    def rPr_with_hlinkClick_bldr(self, rId):
        hlinkClick_bldr = an_hlinkClick().with_rId(rId)
        rPr_bldr = an_rPr().with_nsdecls("a", "r").with_child(hlinkClick_bldr)
        return rPr_bldr

    @pytest.fixture
    def rPr_with_hlinkClick_xml(self, rPr_with_hlinkClick_bldr):
        return rPr_with_hlinkClick_bldr.xml()

    @pytest.fixture
    def rPr_xml(self):
        return an_rPr().with_nsdecls("a", "r").xml()

    @pytest.fixture
    def url(self):
        return "https://github.com/scanny/python-pptx"

    @pytest.fixture
    def url_2(self):
        return "https://pypi.python.org/pypi/python-pptx"


class Describe_Paragraph(object):
    """Unit test suite for pptx.text.text._Paragraph object."""

    def it_can_add_a_line_break(self, line_break_fixture):
        paragraph, expected_xml = line_break_fixture
        paragraph.add_line_break()
        assert paragraph._p.xml == expected_xml

    def it_can_add_a_run(self, paragraph, p_with_r_xml):
        run = paragraph.add_run()
        assert paragraph._p.xml == p_with_r_xml
        assert isinstance(run, _Run)

    def it_knows_its_horizontal_alignment(self, alignment_get_fixture):
        paragraph, expected_value = alignment_get_fixture
        assert paragraph.alignment == expected_value

    def it_can_change_its_horizontal_alignment(self, alignment_set_fixture):
        paragraph, new_value, expected_xml = alignment_set_fixture
        paragraph.alignment = new_value
        assert paragraph._element.xml == expected_xml

    def it_can_clear_itself_of_content(self, clear_fixture):
        paragraph, expected_xml = clear_fixture
        paragraph.clear()
        assert paragraph._element.xml == expected_xml

    def it_provides_access_to_the_default_paragraph_font(self, paragraph, Font_):
        font = paragraph.font
        Font_.assert_called_once_with(paragraph._defRPr)
        assert font == Font_.return_value

    def it_knows_its_indentation_level(self, level_get_fixture):
        paragraph, expected_value = level_get_fixture
        assert paragraph.level == expected_value

    def it_can_change_its_indentation_level(self, level_set_fixture):
        paragraph, new_value, expected_xml = level_set_fixture
        paragraph.level = new_value
        assert paragraph._element.xml == expected_xml

    def it_knows_its_line_spacing(self, spacing_get_fixture):
        paragraph, expected_value = spacing_get_fixture
        assert paragraph.line_spacing == expected_value

    def it_can_change_its_line_spacing(self, spacing_set_fixture):
        paragraph, new_value, expected_xml = spacing_set_fixture
        paragraph.line_spacing = new_value
        assert paragraph._element.xml == expected_xml

    def it_provides_access_to_its_runs(self, runs_fixture):
        paragraph, expected_text = runs_fixture
        runs = paragraph.runs
        assert tuple(r.text for r in runs) == expected_text
        for r in runs:
            assert isinstance(r, _Run)
            assert r._parent == paragraph

    def it_knows_its_space_after(self, after_get_fixture):
        paragraph, expected_value = after_get_fixture
        assert paragraph.space_after == expected_value

    def it_can_change_its_space_after(self, after_set_fixture):
        paragraph, new_value, expected_xml = after_set_fixture
        paragraph.space_after = new_value
        assert paragraph._element.xml == expected_xml

    def it_knows_its_space_before(self, before_get_fixture):
        paragraph, expected_value = before_get_fixture
        assert paragraph.space_before == expected_value

    def it_can_change_its_space_before(self, before_set_fixture):
        paragraph, new_value, expected_xml = before_set_fixture
        paragraph.space_before = new_value
        assert paragraph._element.xml == expected_xml

    def it_knows_what_text_it_contains(self, text_get_fixture):
        p, expected_value = text_get_fixture
        paragraph = _Paragraph(p, None)

        text = paragraph.text

        assert text == expected_value
        assert is_unicode(text)

    def it_can_change_its_text(self, text_set_fixture):
        p, value, expected_xml = text_set_fixture
        paragraph = _Paragraph(p, None)

        paragraph.text = value

        assert paragraph._element.xml == expected_xml

    # fixtures ---------------------------------------------

    @pytest.fixture(
        params=[
            ("a:p", None),
            ("a:p/a:pPr", None),
            ("a:p/a:pPr/a:spcAft/a:spcPct{val=150000}", None),
            ("a:p/a:pPr/a:spcAft/a:spcPts{val=600}", 76200),
        ]
    )
    def after_get_fixture(self, request):
        p_cxml, expected_value = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        return paragraph, expected_value

    @pytest.fixture(
        params=[
            ("a:p", Pt(8.333), "a:p/a:pPr/a:spcAft/a:spcPts{val=833}"),
            (
                "a:p/a:pPr/a:spcAft/a:spcPts{val=600}",
                Pt(42),
                "a:p/a:pPr/a:spcAft/a:spcPts{val=4200}",
            ),
            (
                "a:p/a:pPr/a:spcAft/a:spcPct{val=150000}",
                Pt(24),
                "a:p/a:pPr/a:spcAft/a:spcPts{val=2400}",
            ),
            ("a:p/a:pPr/a:spcAft/a:spcPts{val=600}", None, "a:p/a:pPr"),
        ]
    )
    def after_set_fixture(self, request):
        p_cxml, new_value, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("a:p", None),
            ("a:p/a:pPr{algn=ctr}", PP_ALIGN.CENTER),
            ("a:p/a:pPr{algn=r}", PP_ALIGN.RIGHT),
        ]
    )
    def alignment_get_fixture(self, request):
        p_cxml, expected_value = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        return paragraph, expected_value

    @pytest.fixture(
        params=[
            ("a:p", PP_ALIGN.LEFT, "a:p/a:pPr{algn=l}"),
            ("a:p/a:pPr{algn=l}", PP_ALIGN.JUSTIFY, "a:p/a:pPr{algn=just}"),
            ("a:p/a:pPr{algn=just}", None, "a:p/a:pPr"),
        ]
    )
    def alignment_set_fixture(self, request):
        p_cxml, new_value, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("a:p", None),
            ("a:p/a:pPr", None),
            ("a:p/a:pPr/a:spcBef/a:spcPct{val=150000}", None),
            ("a:p/a:pPr/a:spcBef/a:spcPts{val=600}", 76200),
        ]
    )
    def before_get_fixture(self, request):
        p_cxml, expected_value = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        return paragraph, expected_value

    @pytest.fixture(
        params=[
            ("a:p", Pt(8.333), "a:p/a:pPr/a:spcBef/a:spcPts{val=833}"),
            (
                "a:p/a:pPr/a:spcBef/a:spcPts{val=600}",
                Pt(42),
                "a:p/a:pPr/a:spcBef/a:spcPts{val=4200}",
            ),
            (
                "a:p/a:pPr/a:spcBef/a:spcPct{val=150000}",
                Pt(24),
                "a:p/a:pPr/a:spcBef/a:spcPts{val=2400}",
            ),
            ("a:p/a:pPr/a:spcBef/a:spcPts{val=600}", None, "a:p/a:pPr"),
        ]
    )
    def before_set_fixture(self, request):
        p_cxml, new_value, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, new_value, expected_xml

    @pytest.fixture(
        params=[
            ('a:p/a:r/a:t"foo"', "a:p"),
            ('a:p/(a:br,a:r/a:t"foo")', "a:p"),
            ('a:p/(a:fld,a:br,a:r/a:t"foo")', "a:p"),
        ]
    )
    def clear_fixture(self, request):
        p_cxml, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, expected_xml

    @pytest.fixture(params=[("a:p", 0), ("a:p/a:pPr{lvl=2}", 2)])
    def level_get_fixture(self, request):
        p_cxml, expected_value = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        return paragraph, expected_value

    @pytest.fixture(
        params=[
            ("a:p", 1, "a:p/a:pPr{lvl=1}"),
            ("a:p/a:pPr{lvl=1}", 2, "a:p/a:pPr{lvl=2}"),
            ("a:p/a:pPr{lvl=2}", 0, "a:p/a:pPr"),
        ]
    )
    def level_set_fixture(self, request):
        p_cxml, new_value, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("a:p", "a:p/a:br"),
            ("a:p/a:r", "a:p/(a:r,a:br)"),
            ("a:p/a:br", "a:p/(a:br,a:br)"),
        ]
    )
    def line_break_fixture(self, request):
        cxml, expected_cxml = request.param
        paragraph = _Paragraph(element(cxml), None)
        expected_xml = xml(expected_cxml)
        return paragraph, expected_xml

    @pytest.fixture
    def runs_fixture(self):
        p_cxml = 'a:p/(a:r/a:t"Foo",a:r/a:t"Bar",a:r/a:t"Baz")'
        paragraph = _Paragraph(element(p_cxml), None)
        expected_text = ("Foo", "Bar", "Baz")
        return paragraph, expected_text

    @pytest.fixture(
        params=[
            ("a:p", None),
            ("a:p/a:pPr", None),
            ("a:p/a:pPr/a:lnSpc/a:spcPts{val=1800}", 228600),
            ("a:p/a:pPr/a:lnSpc/a:spcPct{val=142000}", 1.42),
            ("a:p/a:pPr/a:lnSpc/a:spcPct{val=124.64%}", 1.2464),
        ]
    )
    def spacing_get_fixture(self, request):
        p_cxml, expected_value = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        return paragraph, expected_value

    @pytest.fixture(
        params=[
            ("a:p", 1.42, "a:p/a:pPr/a:lnSpc/a:spcPct{val=142000}"),
            ("a:p", Pt(42), "a:p/a:pPr/a:lnSpc/a:spcPts{val=4200}"),
            (
                "a:p/a:pPr/a:lnSpc/a:spcPct{val=110000}",
                0.875,
                "a:p/a:pPr/a:lnSpc/a:spcPct{val=87500}",
            ),
            (
                "a:p/a:pPr/a:lnSpc/a:spcPts{val=600}",
                Pt(42),
                "a:p/a:pPr/a:lnSpc/a:spcPts{val=4200}",
            ),
            (
                "a:p/a:pPr/a:lnSpc/a:spcPts{val=1900}",
                0.925,
                "a:p/a:pPr/a:lnSpc/a:spcPct{val=92500}",
            ),
            (
                "a:p/a:pPr/a:lnSpc/a:spcPct{val=150000}",
                Pt(24),
                "a:p/a:pPr/a:lnSpc/a:spcPts{val=2400}",
            ),
            ("a:p/a:pPr/a:lnSpc/a:spcPts{val=600}", None, "a:p/a:pPr"),
            ("a:p/a:pPr/a:lnSpc/a:spcPct{val=150000}", None, "a:p/a:pPr"),
        ]
    )
    def spacing_set_fixture(self, request):
        p_cxml, new_value, expected_p_cxml = request.param
        paragraph = _Paragraph(element(p_cxml), None)
        expected_xml = xml(expected_p_cxml)
        return paragraph, new_value, expected_xml

    @pytest.fixture(
        params=[
            # ---single-run---
            ('a:p/a:r/a:t"foobar"', "foobar"),
            # ---multiple-runs---
            ('a:p/(a:r/a:t"foo",a:r/a:t"bar")', "foobar"),
            # ---line-break between runs---
            ('a:p/(a:r/a:t"foo",a:br,a:r/a:t"bar")', "foo\vbar"),
            # ---field between runs---
            ('a:p/(a:r/a:t"foo ",a:fld/a:t"42",a:r/a:t" bar")', "foo 42 bar"),
            # ---line-break and field---
            ('a:p/(a:r/a:t" foo",a:br,a:fld/a:t"42")', " foo\v42"),
            # ---other common p child elements included---
            ('a:p/(a:pPr,a:r/a:t"foobar",a:endParaRPr)', "foobar"),
            # ---field by itself---
            ('a:p/a:fld/a:t"42"', "42"),
            # ---line-break by itself---
            ("a:p/a:br", "\v"),
        ]
    )
    def text_get_fixture(self, request):
        p_cxml, expected_value = request.param
        p = element(p_cxml)
        return p, expected_value

    @pytest.fixture(
        params=[
            ('a:p/(a:r/a:t"foo",a:r/a:t"bar")', "foobar", 'a:p/a:r/a:t"foobar"'),
            ("a:p", "", "a:p"),
            ("a:p", "foobar", 'a:p/a:r/a:t"foobar"'),
            ("a:p", "foo\nbar", 'a:p/(a:r/a:t"foo",a:br,a:r/a:t"bar")'),
            ("a:p", "\vfoo\n", 'a:p/(a:br,a:r/a:t"foo",a:br)'),
            ("a:p", "\n\nfoo", 'a:p/(a:br,a:br,a:r/a:t"foo")'),
            ("a:p", "foo\n", 'a:p/(a:r/a:t"foo",a:br)'),
            ("a:p", b"foo\x07\n", 'a:p/(a:r/a:t"foo_x0007_",a:br)'),
            ("a:p", b"7-bit str", 'a:p/a:r/a:t"7-bit str"'),
            ("a:p", b"8-\xc9\x93\xc3\xaf\xc8\xb6 str", 'a:p/a:r/a:t"8-ɓïȶ str"'),
            ("a:p", "ŮŦƑ-8\x1bliteral", 'a:p/a:r/a:t"ŮŦƑ-8_x001B_literal"'),
            (
                "a:p",
                "utf-8 unicode: Hér er texti",
                'a:p/a:r/a:t"utf-8 unicode: Hér er texti"',
            ),
        ]
    )
    def text_set_fixture(self, request):
        p_cxml, value, expected_cxml = request.param
        p = element(p_cxml)
        expected_xml = xml(expected_cxml)
        return p, value, expected_xml

    # fixture components -----------------------------------

    @pytest.fixture
    def Font_(self, request):
        return class_mock(request, "pptx.text.text.Font")

    @pytest.fixture
    def p_bldr(self):
        return a_p().with_nsdecls()

    @pytest.fixture
    def p_with_r_xml(self):
        run_bldr = an_r().with_child(a_t())
        return a_p().with_nsdecls().with_child(run_bldr).xml()

    @pytest.fixture
    def paragraph(self, p_bldr):
        return _Paragraph(p_bldr.element, None)


class Describe_Run(object):
    """Unit-test suite for `pptx.text.text._Run` object."""

    def it_provides_access_to_its_font(self, font_fixture):
        run, rPr, Font_, font_ = font_fixture
        font = run.font
        Font_.assert_called_once_with(rPr)
        assert font == font_

    def it_provides_access_to_a_hyperlink_proxy(self, hyperlink_fixture):
        run, rPr, _Hyperlink_, hlink_ = hyperlink_fixture
        hlink = run.hyperlink
        _Hyperlink_.assert_called_once_with(rPr, run)
        assert hlink is hlink_

    def it_can_get_the_text_of_the_run(self, text_get_fixture):
        run, expected_value = text_get_fixture
        text = run.text
        assert text == expected_value
        assert is_unicode(text)

    def it_can_change_its_text(self, text_set_fixture):
        r, new_value, expected_xml = text_set_fixture
        run = _Run(r, None)

        run.text = new_value

        print("run._r.xml == %s" % repr(run._r.xml))
        print("expected_xml == %s" % repr(expected_xml))
        assert run._r.xml == expected_xml

    # fixtures ---------------------------------------------

    @pytest.fixture
    def font_fixture(self, Font_, font_):
        r = element("a:r/a:rPr")
        rPr = r.rPr
        run = _Run(r, None)
        return run, rPr, Font_, font_

    @pytest.fixture
    def hyperlink_fixture(self, _Hyperlink_, hlink_):
        r = element("a:r/a:rPr")
        rPr = r.rPr
        run = _Run(r, None)
        return run, rPr, _Hyperlink_, hlink_

    @pytest.fixture
    def text_get_fixture(self):
        r = element('a:r/a:t"foobar"')
        run = _Run(r, None)
        return run, "foobar"

    @pytest.fixture(
        params=[
            ("a:r/a:t", "barfoo", 'a:r/a:t"barfoo"'),
            ("a:r/a:t", "bar\x1bfoo", 'a:r/a:t"bar_x001B_foo"'),
            ("a:r/a:t", "bar\tfoo", 'a:r/a:t"bar\tfoo"'),
        ]
    )
    def text_set_fixture(self, request):
        r_cxml, new_value, expected_r_cxml = request.param
        r = element(r_cxml)
        expected_xml = xml(expected_r_cxml)
        return r, new_value, expected_xml

    # fixture components -----------------------------------

    @pytest.fixture
    def Font_(self, request, font_):
        return class_mock(request, "pptx.text.text.Font", return_value=font_)

    @pytest.fixture
    def font_(self, request):
        return instance_mock(request, Font)

    @pytest.fixture
    def _Hyperlink_(self, request, hlink_):
        return class_mock(request, "pptx.text.text._Hyperlink", return_value=hlink_)

    @pytest.fixture
    def hlink_(self, request):
        return instance_mock(request, _Hyperlink)



# encoding: utf-8

"""Embedded Package part objects.

"Package" in this context means another OPC package, i.e. a DOCX, PPTX, or XLSX "file".
"""

from pptx.enum.shapes import PROG_ID
from pptx.opc.constants import CONTENT_TYPE as CT
from pptx.opc.package import Part


class EmbeddedPackagePart(Part):
    """A distinct OPC package, e.g. an Excel file, embedded in this PPTX package.

    Has a partname like: `ppt/embeddings/Microsoft_Excel_Sheet1.xlsx`.
    """

    @classmethod
    def factory(cls, prog_id, object_blob, package):
        """Return a new |EmbeddedPackagePart| subclass instance added to *package*.

        The subclass is determined by `prog_id` which corresponds to the "application"
        used to open the "file-type" of `object_blob`. The returned part contains the
        bytes of `object_blob` and has the content-type also determined by `prog_id`.
        """
        # --- a generic OLE object has no subclass ---
        if prog_id not in PROG_ID:
            return cls(
                package.next_partname("/ppt/embeddings/oleObject%d.bin"),
                CT.OFC_OLE_OBJECT,
                object_blob,
                package,
            )

        # --- A Microsoft Office file-type is a distinguished package object ---
        EmbeddedPartCls = {
            PROG_ID.DOCX: EmbeddedDocxPart,
            PROG_ID.PPTX: EmbeddedPptxPart,
            PROG_ID.XLSX: EmbeddedXlsxPart,
        }[prog_id]

        return EmbeddedPartCls.new(object_blob, package)

    @classmethod
    def new(cls, blob, package):
        """Return new |EmbeddedPackagePart| subclass object.

        The returned part object contains `blob` and is added to `package`.
        """
        partname = package.next_partname(cls.partname_template)
        return cls(partname, cls.content_type, blob, package)


class EmbeddedDocxPart(EmbeddedPackagePart):
    """A Word .docx file stored in a part.

    This part-type arises when a Word document appears as an embedded OLE-object shape.
    """

    partname_template = "/ppt/embeddings/Microsoft_Word_Document%d.docx"
    content_type = CT.WML_DOCUMENT


class EmbeddedPptxPart(EmbeddedPackagePart):
    """A PowerPoint file stored in a part.

    This part-type arises when a PowerPoint presentation (.pptx file) appears as an
    embedded OLE-object shape.
    """

    partname_template = "/ppt/embeddings/Microsoft_PowerPoint_Presentation%d.pptx"
    content_type = CT.PML_PRESENTATION


class EmbeddedXlsxPart(EmbeddedPackagePart):
    """An Excel file stored in a part.

    This part-type arises as the data source for a chart, but may also be the OLE-object
    for an embedded object shape.
    """

    partname_template = "/ppt/embeddings/Microsoft_Excel_Sheet%d.xlsx"
    content_type = CT.SML_SHEET



# encoding: utf-8

"""
Enumerations used by charts and related objects
"""

from __future__ import absolute_import

from .base import (
    alias,
    Enumeration,
    EnumMember,
    ReturnValueOnlyEnumMember,
    XmlEnumeration,
    XmlMappedEnumMember,
)


class XL_AXIS_CROSSES(XmlEnumeration):
    """
    Specifies the point on the specified axis where the other axis crosses.

    Example::

        from pptx.enum.chart import XL_AXIS_CROSSES

        value_axis.crosses = XL_AXIS_CROSSES.MAXIMUM
    """

    __ms_name__ = "XlAxisCrosses"

    __url__ = "https://msdn.microsoft.com/en-us/library/office/ff745402.aspx"

    __members__ = (
        XmlMappedEnumMember(
            "AUTOMATIC",
            -4105,
            "autoZero",
            "The axis crossing point is set " "automatically, often at zero.",
        ),
        ReturnValueOnlyEnumMember(
            "CUSTOM",
            -4114,
            "The .crosses_at property specifies the axis cr" "ossing point.",
        ),
        XmlMappedEnumMember(
            "MAXIMUM", 2, "max", "The axis crosses at the maximum value."
        ),
        XmlMappedEnumMember(
            "MINIMUM", 4, "min", "The axis crosses at the minimum value."
        ),
    )


class XL_CATEGORY_TYPE(Enumeration):
    """
    Specifies the type of the category axis.

    Example::

        from pptx.enum.chart import XL_CATEGORY_TYPE

        date_axis = chart.category_axis
        assert date_axis.category_type == XL_CATEGORY_TYPE.TIME_SCALE
    """

    __ms_name__ = "XlCategoryType"

    __url__ = "https://msdn.microsoft.com/EN-US/library/office/ff746136.aspx"

    __members__ = (
        EnumMember(
            "AUTOMATIC_SCALE", -4105, "The application controls the axis " "type."
        ),
        EnumMember(
            "CATEGORY_SCALE", 2, "Axis groups data by an arbitrary set of " "categories"
        ),
        EnumMember(
            "TIME_SCALE",
            3,
            "Axis groups data on a time scale of days, " "months, or years.",
        ),
    )


class XL_CHART_TYPE(Enumeration):
    """
    Specifies the type of a chart.

    Example::

        from pptx.enum.chart import XL_CHART_TYPE

        assert chart.chart_type == XL_CHART_TYPE.BAR_STACKED
    """

    __ms_name__ = "XlChartType"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff838409.aspx"

    __members__ = (
        EnumMember("THREE_D_AREA", -4098, "3D Area."),
        EnumMember("THREE_D_AREA_STACKED", 78, "3D Stacked Area."),
        EnumMember("THREE_D_AREA_STACKED_100", 79, "100% Stacked Area."),
        EnumMember("THREE_D_BAR_CLUSTERED", 60, "3D Clustered Bar."),
        EnumMember("THREE_D_BAR_STACKED", 61, "3D Stacked Bar."),
        EnumMember("THREE_D_BAR_STACKED_100", 62, "3D 100% Stacked Bar."),
        EnumMember("THREE_D_COLUMN", -4100, "3D Column."),
        EnumMember("THREE_D_COLUMN_CLUSTERED", 54, "3D Clustered Column."),
        EnumMember("THREE_D_COLUMN_STACKED", 55, "3D Stacked Column."),
        EnumMember("THREE_D_COLUMN_STACKED_100", 56, "3D 100% Stacked Column."),
        EnumMember("THREE_D_LINE", -4101, "3D Line."),
        EnumMember("THREE_D_PIE", -4102, "3D Pie."),
        EnumMember("THREE_D_PIE_EXPLODED", 70, "Exploded 3D Pie."),
        EnumMember("AREA", 1, "Area"),
        EnumMember("AREA_STACKED", 76, "Stacked Area."),
        EnumMember("AREA_STACKED_100", 77, "100% Stacked Area."),
        EnumMember("BAR_CLUSTERED", 57, "Clustered Bar."),
        EnumMember("BAR_OF_PIE", 71, "Bar of Pie."),
        EnumMember("BAR_STACKED", 58, "Stacked Bar."),
        EnumMember("BAR_STACKED_100", 59, "100% Stacked Bar."),
        EnumMember("BUBBLE", 15, "Bubble."),
        EnumMember("BUBBLE_THREE_D_EFFECT", 87, "Bubble with 3D effects."),
        EnumMember("COLUMN_CLUSTERED", 51, "Clustered Column."),
        EnumMember("COLUMN_STACKED", 52, "Stacked Column."),
        EnumMember("COLUMN_STACKED_100", 53, "100% Stacked Column."),
        EnumMember("CONE_BAR_CLUSTERED", 102, "Clustered Cone Bar."),
        EnumMember("CONE_BAR_STACKED", 103, "Stacked Cone Bar."),
        EnumMember("CONE_BAR_STACKED_100", 104, "100% Stacked Cone Bar."),
        EnumMember("CONE_COL", 105, "3D Cone Column."),
        EnumMember("CONE_COL_CLUSTERED", 99, "Clustered Cone Column."),
        EnumMember("CONE_COL_STACKED", 100, "Stacked Cone Column."),
        EnumMember("CONE_COL_STACKED_100", 101, "100% Stacked Cone Column."),
        EnumMember("CYLINDER_BAR_CLUSTERED", 95, "Clustered Cylinder Bar."),
        EnumMember("CYLINDER_BAR_STACKED", 96, "Stacked Cylinder Bar."),
        EnumMember("CYLINDER_BAR_STACKED_100", 97, "100% Stacked Cylinder Bar."),
        EnumMember("CYLINDER_COL", 98, "3D Cylinder Column."),
        EnumMember("CYLINDER_COL_CLUSTERED", 92, "Clustered Cone Column."),
        EnumMember("CYLINDER_COL_STACKED", 93, "Stacked Cone Column."),
        EnumMember("CYLINDER_COL_STACKED_100", 94, "100% Stacked Cylinder Column."),
        EnumMember("DOUGHNUT", -4120, "Doughnut."),
        EnumMember("DOUGHNUT_EXPLODED", 80, "Exploded Doughnut."),
        EnumMember("LINE", 4, "Line."),
        EnumMember("LINE_MARKERS", 65, "Line with Markers."),
        EnumMember("LINE_MARKERS_STACKED", 66, "Stacked Line with Markers."),
        EnumMember("LINE_MARKERS_STACKED_100", 67, "100% Stacked Line with Markers."),
        EnumMember("LINE_STACKED", 63, "Stacked Line."),
        EnumMember("LINE_STACKED_100", 64, "100% Stacked Line."),
        EnumMember("PIE", 5, "Pie."),
        EnumMember("PIE_EXPLODED", 69, "Exploded Pie."),
        EnumMember("PIE_OF_PIE", 68, "Pie of Pie."),
        EnumMember("PYRAMID_BAR_CLUSTERED", 109, "Clustered Pyramid Bar."),
        EnumMember("PYRAMID_BAR_STACKED", 110, "Stacked Pyramid Bar."),
        EnumMember("PYRAMID_BAR_STACKED_100", 111, "100% Stacked Pyramid Bar."),
        EnumMember("PYRAMID_COL", 112, "3D Pyramid Column."),
        EnumMember("PYRAMID_COL_CLUSTERED", 106, "Clustered Pyramid Column."),
        EnumMember("PYRAMID_COL_STACKED", 107, "Stacked Pyramid Column."),
        EnumMember("PYRAMID_COL_STACKED_100", 108, "100% Stacked Pyramid Column."),
        EnumMember("RADAR", -4151, "Radar."),
        EnumMember("RADAR_FILLED", 82, "Filled Radar."),
        EnumMember("RADAR_MARKERS", 81, "Radar with Data Markers."),
        EnumMember("STOCK_HLC", 88, "High-Low-Close."),
        EnumMember("STOCK_OHLC", 89, "Open-High-Low-Close."),
        EnumMember("STOCK_VHLC", 90, "Volume-High-Low-Close."),
        EnumMember("STOCK_VOHLC", 91, "Volume-Open-High-Low-Close."),
        EnumMember("SURFACE", 83, "3D Surface."),
        EnumMember("SURFACE_TOP_VIEW", 85, "Surface (Top View)."),
        EnumMember("SURFACE_TOP_VIEW_WIREFRAME", 86, "Surface (Top View wireframe)."),
        EnumMember("SURFACE_WIREFRAME", 84, "3D Surface (wireframe)."),
        EnumMember("XY_SCATTER", -4169, "Scatter."),
        EnumMember("XY_SCATTER_LINES", 74, "Scatter with Lines."),
        EnumMember(
            "XY_SCATTER_LINES_NO_MARKERS",
            75,
            "Scatter with Lines and No Da" "ta Markers.",
        ),
        EnumMember("XY_SCATTER_SMOOTH", 72, "Scatter with Smoothed Lines."),
        EnumMember(
            "XY_SCATTER_SMOOTH_NO_MARKERS",
            73,
            "Scatter with Smoothed Lines" " and No Data Markers.",
        ),
    )


@alias("XL_LABEL_POSITION")
class XL_DATA_LABEL_POSITION(XmlEnumeration):
    """
    Specifies where the data label is positioned.

    Example::

        from pptx.enum.chart import XL_LABEL_POSITION

        data_labels = chart.plots[0].data_labels
        data_labels.position = XL_LABEL_POSITION.OUTSIDE_END
    """

    __ms_name__ = "XlDataLabelPosition"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff745082.aspx"

    __members__ = (
        XmlMappedEnumMember(
            "ABOVE", 0, "t", "The data label is positioned above the data point."
        ),
        XmlMappedEnumMember(
            "BELOW", 1, "b", "The data label is positioned below the data point."
        ),
        XmlMappedEnumMember(
            "BEST_FIT", 5, "bestFit", "Word sets the position of the data label."
        ),
        XmlMappedEnumMember(
            "CENTER",
            -4108,
            "ctr",
            "The data label is centered on the data point or inside a bar or a pie "
            "slice.",
        ),
        XmlMappedEnumMember(
            "INSIDE_BASE",
            4,
            "inBase",
            "The data label is positioned inside the data point at the bottom edge.",
        ),
        XmlMappedEnumMember(
            "INSIDE_END",
            3,
            "inEnd",
            "The data label is positioned inside the data point at the top edge.",
        ),
        XmlMappedEnumMember(
            "LEFT",
            -4131,
            "l",
            "The data label is positioned to the left of the data point.",
        ),
        ReturnValueOnlyEnumMember("MIXED", 6, "Data labels are in multiple positions."),
        XmlMappedEnumMember(
            "OUTSIDE_END",
            2,
            "outEnd",
            "The data label is positioned outside the data point at the top edge.",
        ),
        XmlMappedEnumMember(
            "RIGHT",
            -4152,
            "r",
            "The data label is positioned to the right of the data point.",
        ),
    )


class XL_LEGEND_POSITION(XmlEnumeration):
    """
    Specifies the position of the legend on a chart.

    Example::

        from pptx.enum.chart import XL_LEGEND_POSITION

        chart.has_legend = True
        chart.legend.position = XL_LEGEND_POSITION.BOTTOM
    """

    __ms_name__ = "XlLegendPosition"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff745840.aspx"

    __members__ = (
        XmlMappedEnumMember("BOTTOM", -4107, "b", "Below the chart."),
        XmlMappedEnumMember(
            "CORNER", 2, "tr", "In the upper-right corner of the chart borde" "r."
        ),
        ReturnValueOnlyEnumMember("CUSTOM", -4161, "A custom position."),
        XmlMappedEnumMember("LEFT", -4131, "l", "Left of the chart."),
        XmlMappedEnumMember("RIGHT", -4152, "r", "Right of the chart."),
        XmlMappedEnumMember("TOP", -4160, "t", "Above the chart."),
    )


class XL_MARKER_STYLE(XmlEnumeration):
    """
    Specifies the marker style for a point or series in a line chart, scatter
    chart, or radar chart.

    Example::

        from pptx.enum.chart import XL_MARKER_STYLE

        series.marker.style = XL_MARKER_STYLE.CIRCLE
    """

    __ms_name__ = "XlMarkerStyle"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff197219.aspx"

    __members__ = (
        XmlMappedEnumMember("AUTOMATIC", -4105, "auto", "Automatic markers"),
        XmlMappedEnumMember("CIRCLE", 8, "circle", "Circular markers"),
        XmlMappedEnumMember("DASH", -4115, "dash", "Long bar markers"),
        XmlMappedEnumMember("DIAMOND", 2, "diamond", "Diamond-shaped markers"),
        XmlMappedEnumMember("DOT", -4118, "dot", "Short bar markers"),
        XmlMappedEnumMember("NONE", -4142, "none", "No markers"),
        XmlMappedEnumMember("PICTURE", -4147, "picture", "Picture markers"),
        XmlMappedEnumMember("PLUS", 9, "plus", "Square markers with a plus sign"),
        XmlMappedEnumMember("SQUARE", 1, "square", "Square markers"),
        XmlMappedEnumMember("STAR", 5, "star", "Square markers with an  asterisk"),
        XmlMappedEnumMember("TRIANGLE", 3, "triangle", "Triangular markers"),
        XmlMappedEnumMember("X", -4168, "x", "Square markers with an X"),
    )


class XL_TICK_MARK(XmlEnumeration):
    """
    Specifies a type of axis tick for a chart.

    Example::

        from pptx.enum.chart import XL_TICK_MARK

        chart.value_axis.minor_tick_mark = XL_TICK_MARK.INSIDE
    """

    __ms_name__ = "XlTickMark"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff193878.aspx"

    __members__ = (
        XmlMappedEnumMember("CROSS", 4, "cross", "Tick mark crosses the axis"),
        XmlMappedEnumMember("INSIDE", 2, "in", "Tick mark appears inside the axis"),
        XmlMappedEnumMember("NONE", -4142, "none", "No tick mark"),
        XmlMappedEnumMember("OUTSIDE", 3, "out", "Tick mark appears outside the axis"),
    )


class XL_TICK_LABEL_POSITION(XmlEnumeration):
    """
    Specifies the position of tick-mark labels on a chart axis.

    Example::

        from pptx.enum.chart import XL_TICK_LABEL_POSITION

        category_axis = chart.category_axis
        category_axis.tick_label_position = XL_TICK_LABEL_POSITION.LOW
    """

    __ms_name__ = "XlTickLabelPosition"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff822561.aspx"

    __members__ = (
        XmlMappedEnumMember("HIGH", -4127, "high", "Top or right side of the chart."),
        XmlMappedEnumMember("LOW", -4134, "low", "Bottom or left side of the chart."),
        XmlMappedEnumMember(
            "NEXT_TO_AXIS",
            4,
            "nextTo",
            "Next to axis (where axis is not at" " either side of the chart).",
        ),
        XmlMappedEnumMember("NONE", -4142, "none", "No tick labels."),
    )



# encoding: utf-8

"""Test suite for pptx.shapes.picture module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.line import LineFormat
from pptx.enum.shapes import MSO_SHAPE, MSO_SHAPE_TYPE, PP_MEDIA_TYPE
from pptx.parts.image import Image
from pptx.parts.slide import SlidePart
from pptx.shapes.picture import _BasePicture, _MediaFormat, Movie, Picture
from pptx.util import Pt

from ..unitutil.cxml import element, xml
from ..unitutil.mock import call, class_mock, instance_mock, property_mock


class Describe_BasePicture(object):
    def it_knows_its_cropping(self, crop_get_fixture):
        picture, prop_name, expected_value = crop_get_fixture
        crop = getattr(picture, prop_name)
        assert abs(crop - expected_value) < 0.000001

    def it_can_change_its_cropping(self, crop_set_fixture):
        picture, prop_name, value, expected_xml = crop_set_fixture
        setattr(picture, prop_name, value)
        assert picture._element.xml == expected_xml

    def it_provides_access_to_its_outline(self, line_fixture):
        picture = line_fixture
        line = picture.line
        assert isinstance(line, LineFormat)
        # exercise line to test has_line interface, .ln and .get_or_add_ln()
        picture.line.width = Pt(1)
        assert picture.line.width == Pt(1)

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("p:pic/p:blipFill", "left", 0.0),
            ("p:pic/p:blipFill/a:srcRect", "top", 0.0),
            ("p:pic/p:blipFill/a:srcRect{l=99999}", "bottom", 0.0),
            ("p:pic/p:blipFill/a:srcRect{l=42424}", "left", 0.42424),
            ("p:pic/p:blipFill/a:srcRect{t=-10000}", "top", -0.1),
            ("p:pic/p:blipFill/a:srcRect{r=250000}", "right", 2.5),
            ("p:pic/p:blipFill/a:srcRect{b=33333}", "bottom", 0.33333),
        ]
    )
    def crop_get_fixture(self, request):
        pic_cxml, side, expected_value = request.param
        picture = Picture(element(pic_cxml), None)
        prop_name = "crop_%s" % side
        return picture, prop_name, expected_value

    @pytest.fixture(
        params=[
            (
                "p:pic{a:b=c}/p:blipFill",
                "left",
                0.11,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{l=11000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill",
                "top",
                0.21,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{t=21000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill",
                "right",
                0.31,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{r=31000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill",
                "bottom",
                0.41,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{b=41000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{l=80000}",
                "left",
                0.21,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{l=21000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{t=70000}",
                "top",
                0.22,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{t=22000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{r=60000}",
                "right",
                0.23,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{r=23000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{b=50000}",
                "bottom",
                0.24,
                "p:pic{a:b=c}/p:blipFill/a:srcRect{b=24000}",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{l=90000}",
                "left",
                0,
                "p:pic{a:b=c}/p:blipFill/a:srcRect",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{t=91000}",
                "top",
                0.0,
                "p:pic{a:b=c}/p:blipFill/a:srcRect",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{r=92000}",
                "right",
                0,
                "p:pic{a:b=c}/p:blipFill/a:srcRect",
            ),
            (
                "p:pic{a:b=c}/p:blipFill/a:srcRect{b=93000}",
                "bottom",
                0.0,
                "p:pic{a:b=c}/p:blipFill/a:srcRect",
            ),
        ]
    )
    def crop_set_fixture(self, request):
        pic_cxml, side, value, expected_cxml = request.param
        pic = element(pic_cxml)
        picture = Picture(pic, None)
        prop_name = "crop_%s" % side
        expected_xml = xml(expected_cxml)
        return picture, prop_name, value, expected_xml

    @pytest.fixture
    def line_fixture(self):
        return _BasePicture(element("p:pic/p:spPr"), None)


class DescribeMovie(object):
    def it_knows_its_shape_type(self, shape_type_fixture):
        movie = shape_type_fixture
        assert movie.shape_type == MSO_SHAPE_TYPE.MEDIA

    def it_knows_its_media_type(self, media_type_fixture):
        movie = media_type_fixture
        assert movie.media_type == PP_MEDIA_TYPE.MOVIE

    def it_provides_access_to_its_media_format(self, format_fixture):
        movie, MediaFormat_, pic, parent, media_format_ = format_fixture
        media_format = movie.media_format
        MediaFormat_.assert_called_once_with(pic, parent)
        assert media_format is media_format_

    def it_provides_access_to_its_poster_frame_image(self, pfrm_fixture):
        movie, slide_part_, calls, expected_value = pfrm_fixture
        poster_frame = movie.poster_frame
        assert slide_part_.get_image.call_args_list == calls
        assert poster_frame == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def format_fixture(self, _MediaFormat_, media_format_):
        pic = element("p:pic")
        parent = movie = Movie(pic, None)
        return movie, _MediaFormat_, pic, parent, media_format_

    @pytest.fixture
    def media_type_fixture(self):
        return Movie(None, None)

    @pytest.fixture(
        params=[
            ("p:pic/p:blipFill/a:blip{r:embed=rId42}", True),
            ("p:pic/p:blipFill/a:blip", False),
        ]
    )
    def pfrm_fixture(self, request, slide_part_, image_, part_prop_):
        cxml, is_present = request.param
        movie = Movie(element(cxml), None)
        calls, expected_value = [], None
        part_prop_.return_value = slide_part_
        slide_part_.get_image.return_value = image_
        if is_present:
            calls.append(call("rId42"))
            expected_value = image_
        return movie, slide_part_, calls, expected_value

    @pytest.fixture
    def shape_type_fixture(self):
        return Movie(None, None)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def image_(self, request):
        return instance_mock(request, Image)

    @pytest.fixture
    def _MediaFormat_(self, request, media_format_):
        return class_mock(
            request, "pptx.shapes.picture._MediaFormat", return_value=media_format_
        )

    @pytest.fixture
    def media_format_(self, request):
        return instance_mock(request, _MediaFormat)

    @pytest.fixture
    def part_prop_(self, request):
        return property_mock(request, Movie, "part")

    @pytest.fixture
    def slide_part_(self, request):
        return instance_mock(request, SlidePart)


class DescribePicture(object):
    def it_knows_its_masking_shape(self, autoshape_get_fixture):
        picture, expected_value = autoshape_get_fixture
        auto_shape_type = picture.auto_shape_type
        assert auto_shape_type == expected_value

    def it_can_change_its_masking_shape(self, autoshape_set_fixture):
        picture, new_value, expected_xml = autoshape_set_fixture
        picture.auto_shape_type = new_value
        assert picture._element.xml == expected_xml

    def it_knows_its_shape_type(self, shape_type_fixture):
        picture = shape_type_fixture
        assert picture.shape_type == MSO_SHAPE_TYPE.PICTURE

    def it_provides_access_to_its_image(self, image_fixture):
        picture, slide_part_, rId, image_ = image_fixture
        image = picture.image
        slide_part_.get_image.assert_called_once_with(rId)
        assert image is image_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("", None),
            ("/a:prstGeom{prst=rect}", MSO_SHAPE.RECTANGLE),
            ("/a:prstGeom{prst=hexagon}", MSO_SHAPE.HEXAGON),
        ]
    )
    def autoshape_get_fixture(self, request):
        prstGeom_cxml, expected_value = request.param
        pic_cxml = "p:pic/p:spPr%s" % prstGeom_cxml
        picture = Picture(element(pic_cxml), None)
        return picture, expected_value

    @pytest.fixture(
        params=[
            (
                "p:pic/p:spPr/a:custGeom",
                MSO_SHAPE.RECTANGLE,
                "p:pic/p:spPr/a:prstGeom{prst=rect}",
            ),
            (
                "p:pic/p:spPr/a:prstGeom{prst=rect}",
                MSO_SHAPE.OVAL,
                "p:pic/p:spPr/a:prstGeom{prst=ellipse}",
            ),
            (
                "p:pic/p:spPr/a:prstGeom{prst=ellipse}",
                MSO_SHAPE.PIE,
                "p:pic/p:spPr/a:prstGeom{prst=pie}",
            ),
        ]
    )
    def autoshape_set_fixture(self, request):
        pic_cxml, new_value, expected_cxml = request.param
        picture = Picture(element(pic_cxml), None)
        expected_xml = xml(expected_cxml)
        return picture, new_value, expected_xml

    @pytest.fixture
    def image_fixture(self, part_prop_, slide_part_, image_):
        pic_cxml, rId = "p:pic/p:blipFill/a:blip{r:embed=rId42}", "rId42"
        picture = Picture(element(pic_cxml), None)
        slide_part_.get_image.return_value = image_
        return picture, slide_part_, rId, image_

    @pytest.fixture
    def shape_type_fixture(self):
        return Picture(None, None)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def image_(self, request):
        return instance_mock(request, Image)

    @pytest.fixture
    def part_prop_(self, request, slide_part_):
        return property_mock(request, Picture, "part", return_value=slide_part_)

    @pytest.fixture
    def slide_part_(self, request):
        return instance_mock(request, SlidePart)



# encoding: utf-8

"""
Test suite for the pptx.opc.packuri module
"""

import pytest

from pptx.opc.packuri import PackURI


class DescribePackURI(object):
    def cases(self, expected_values):
        """
        Return list of tuples zipped from uri_str cases and
        *expected_values*. Raise if lengths don't match.
        """
        uri_str_cases = ["/", "/ppt/presentation.xml", "/ppt/slides/slide1.xml"]
        if len(expected_values) != len(uri_str_cases):
            msg = "len(expected_values) differs from len(uri_str_cases)"
            raise AssertionError(msg)
        pack_uris = [PackURI(uri_str) for uri_str in uri_str_cases]
        return zip(pack_uris, expected_values)

    def it_can_construct_from_relative_ref(self):
        baseURI = "/ppt/slides"
        relative_ref = "../slideLayouts/slideLayout1.xml"
        pack_uri = PackURI.from_rel_ref(baseURI, relative_ref)
        assert pack_uri == "/ppt/slideLayouts/slideLayout1.xml"

    def it_should_raise_on_construct_with_bad_pack_uri_str(self):
        with pytest.raises(ValueError):
            PackURI("foobar")

    def it_can_calculate_baseURI(self):
        expected_values = ("/", "/ppt", "/ppt/slides")
        for pack_uri, expected_baseURI in self.cases(expected_values):
            assert pack_uri.baseURI == expected_baseURI

    def it_can_calculate_extension(self):
        expected_values = ("", "xml", "xml")
        for pack_uri, expected_ext in self.cases(expected_values):
            assert pack_uri.ext == expected_ext

    def it_can_calculate_filename(self):
        expected_values = ("", "presentation.xml", "slide1.xml")
        for pack_uri, expected_filename in self.cases(expected_values):
            assert pack_uri.filename == expected_filename

    def it_knows_the_filename_index(self):
        expected_values = (None, None, 1)
        for pack_uri, expected_idx in self.cases(expected_values):
            assert pack_uri.idx == expected_idx

    def it_can_calculate_membername(self):
        expected_values = ("", "ppt/presentation.xml", "ppt/slides/slide1.xml")
        for pack_uri, expected_membername in self.cases(expected_values):
            assert pack_uri.membername == expected_membername

    def it_can_calculate_relative_ref_value(self):
        cases = (
            ("/", "/ppt/presentation.xml", "ppt/presentation.xml"),
            (
                "/ppt",
                "/ppt/slideMasters/slideMaster1.xml",
                "slideMasters/slideMaster1.xml",
            ),
            (
                "/ppt/slides",
                "/ppt/slideLayouts/slideLayout1.xml",
                "../slideLayouts/slideLayout1.xml",
            ),
        )
        for baseURI, uri_str, expected_relative_ref in cases:
            pack_uri = PackURI(uri_str)
            assert pack_uri.relative_ref(baseURI) == expected_relative_ref

    def it_can_calculate_rels_uri(self):
        expected_values = (
            "/_rels/.rels",
            "/ppt/_rels/presentation.xml.rels",
            "/ppt/slides/_rels/slide1.xml.rels",
        )
        for pack_uri, expected_rels_uri in self.cases(expected_values):
            assert pack_uri.rels_uri == expected_rels_uri



# encoding: utf-8

"""
Test suite for opc.oxml module
"""

from __future__ import absolute_import, print_function, unicode_literals

import pytest

from pptx.opc.constants import RELATIONSHIP_TARGET_MODE as RTM
from pptx.opc.oxml import (
    CT_Default,
    CT_Override,
    CT_Relationship,
    CT_Relationships,
    CT_Types,
    oxml_tostring,
    serialize_part_xml,
)
from pptx.oxml import parse_xml

from .unitdata.rels import (
    a_Default,
    an_Override,
    a_Relationship,
    a_Relationships,
    a_Types,
)


class DescribeCT_Default(object):
    def it_provides_read_access_to_xml_values(self):
        default = a_Default().element
        assert default.extension == "xml"
        assert default.contentType == "application/xml"


class DescribeCT_Override(object):
    def it_provides_read_access_to_xml_values(self):
        override = an_Override().element
        assert override.partName == "/part/name.xml"
        assert override.contentType == "app/vnd.type"


class DescribeCT_Relationship(object):
    def it_provides_read_access_to_xml_values(self):
        rel = a_Relationship().element
        assert rel.rId == "rId9"
        assert rel.reltype == "ReLtYpE"
        assert rel.target_ref == "docProps/core.xml"
        assert rel.targetMode == RTM.INTERNAL

    def it_can_construct_from_attribute_values(self):
        cases = (
            ("rId9", "ReLtYpE", "foo/bar.xml", None),
            ("rId9", "ReLtYpE", "bar/foo.xml", RTM.INTERNAL),
            ("rId9", "ReLtYpE", "http://some/link", RTM.EXTERNAL),
        )
        for rId, reltype, target, target_mode in cases:
            if target_mode is None:
                rel = CT_Relationship.new(rId, reltype, target)
            else:
                rel = CT_Relationship.new(rId, reltype, target, target_mode)
            builder = a_Relationship().with_target(target)
            if target_mode == RTM.EXTERNAL:
                builder = builder.with_target_mode(RTM.EXTERNAL)
            expected_rel_xml = builder.xml
            assert rel.xml == expected_rel_xml


class DescribeCT_Relationships(object):
    def it_can_construct_a_new_relationships_element(self):
        rels = CT_Relationships.new()
        expected_xml = (
            "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n"
            '<Relationships xmlns="http://schemas.openxmlformats.org/package'
            '/2006/relationships"/>'
        )
        assert rels.xml.decode("utf-8") == expected_xml

    def it_can_build_rels_element_incrementally(self):
        # setup ------------------------
        rels = CT_Relationships.new()
        # exercise ---------------------
        rels.add_rel("rId1", "http://reltype1", "docProps/core.xml")
        rels.add_rel("rId2", "http://linktype", "http://some/link", True)
        rels.add_rel("rId3", "http://reltype2", "../slides/slide1.xml")
        # verify -----------------------
        expected_rels_xml = a_Relationships().xml
        actual_xml = oxml_tostring(rels, encoding="unicode", pretty_print=True)
        assert actual_xml == expected_rels_xml

    def it_can_generate_rels_file_xml(self):
        expected_xml = (
            "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n"
            '<Relationships xmlns="http://schemas.openxmlformats.org/package'
            '/2006/relationships"/>'.encode("utf-8")
        )
        assert CT_Relationships.new().xml == expected_xml


class DescribeCT_Types(object):
    def it_provides_access_to_default_child_elements(self):
        types = a_Types().element
        assert len(types.default_lst) == 2
        for default in types.default_lst:
            assert isinstance(default, CT_Default)

    def it_provides_access_to_override_child_elements(self):
        types = a_Types().element
        assert len(types.override_lst) == 3
        for override in types.override_lst:
            assert isinstance(override, CT_Override)

    def it_should_have_empty_list_on_no_matching_elements(self):
        types = a_Types().empty().element
        assert types.default_lst == []
        assert types.override_lst == []

    def it_can_construct_a_new_types_element(self):
        types = CT_Types.new()
        expected_xml = a_Types().empty().xml
        assert types.xml == expected_xml

    def it_can_build_types_element_incrementally(self):
        types = CT_Types.new()
        types.add_default("xml", "application/xml")
        types.add_default("jpeg", "image/jpeg")
        types.add_override("/docProps/core.xml", "app/vnd.type1")
        types.add_override("/ppt/presentation.xml", "app/vnd.type2")
        types.add_override("/docProps/thumbnail.jpeg", "image/jpeg")
        expected_types_xml = a_Types().xml
        assert types.xml == expected_types_xml


class DescribeSerializePartXml(object):
    def it_produces_properly_formatted_xml_for_an_opc_part(
        self, part_elm, expected_part_xml
    ):
        """
        Tested aspects:
        ---------------
        * [X] it generates an XML declaration
        * [X] it produces no whitespace between elements
        * [X] it preserves unused namespaces
        * [X] it returns bytes ready to save to file (not unicode)
        """
        xml = serialize_part_xml(part_elm)
        assert xml == expected_part_xml
        # xml contains 134 chars, of which 3 are double-byte; it will have
        # len of 134 if it's unicode and 137 if it's bytes
        assert len(xml) == 137

    # fixtures -----------------------------------

    @pytest.fixture
    def part_elm(self):
        return parse_xml(
            '<f:foo xmlns:f="http://foo" xmlns:b="http://bar">\n  <f:bar>fØØ'
            "bÅr</f:bar>\n</f:foo>\n"
        )

    @pytest.fixture
    def expected_part_xml(self):
        unicode_xml = (
            "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n"
            '<f:foo xmlns:f="http://foo" xmlns:b="http://bar"><f:bar>fØØbÅr<'
            "/f:bar></f:foo>"
        )
        xml_bytes = unicode_xml.encode("utf-8")
        return xml_bytes



# encoding: utf-8

"""
Helper methods and variables for acceptance tests.
"""

import os


def absjoin(*paths):
    return os.path.abspath(os.path.join(*paths))


thisdir = os.path.split(__file__)[0]
scratch_dir = absjoin(thisdir, "../_scratch")
# new ones should go here instead, others should be moved over
test_pptx_dir = absjoin(thisdir, "test_files")

# legacy test pptx files ---------------
no_core_props_pptx_path = absjoin(
    thisdir, "../../tests/test_files", "no-core-props.pptx"
)

# scratch test pptx file ---------------
saved_pptx_path = absjoin(scratch_dir, "test_out.pptx")

test_text = "python-pptx was here!"


def cls_qname(obj):
    module_name = obj.__module__
    cls_name = obj.__class__.__name__
    qname = "%s.%s" % (module_name, cls_name)
    return qname


def count(start=0, step=1):
    """
    Local implementation of `itertools.count()` to allow v2.6 compatibility.
    """
    n = start
    while True:
        yield n
        n += step


def test_file(filename):
    """
    Return the absolute path to the file having *filename* in acceptance
    test_files directory.
    """
    return absjoin(thisdir, "test_files", filename)


def test_image(filename):
    """
    Return the absolute path to image file having *filename* in test_files
    directory.
    """
    return absjoin(thisdir, "test_files", filename)


def test_pptx(name):
    """
    Return the absolute path to test .pptx file with root name *name*.
    """
    return absjoin(thisdir, "test_files", "%s.pptx" % name)



# encoding: utf-8

"""
Gherkin step implementations for placeholder-related features.
"""

from __future__ import absolute_import

import hashlib

from behave import given, when, then

from pptx import Presentation
from pptx.chart.data import CategoryChartData
from pptx.enum.chart import XL_CHART_TYPE
from pptx.enum.shapes import MSO_SHAPE_TYPE, PP_PLACEHOLDER
from pptx.shapes.base import _PlaceholderFormat

from helpers import saved_pptx_path, test_file, test_pptx, test_text


# given ===================================================


@given("a bullet body placeholder")
def given_a_bullet_body_placeholder(context):
    prs = Presentation(test_pptx("ph-unpopulated-placeholders"))
    context.prs = prs
    context.sld = prs.slides[2]
    context.body = prs.slides[2].shapes.placeholders[10]


@given("a known {placeholder_type} placeholder shape")
def given_a_known_placeholder_shape(context, placeholder_type):
    context.execute_steps(
        "given an unpopulated %s placeholder shape" % placeholder_type
    )


@given("a layout placeholder having directly set position and size")
def given_layout_placeholder_with_directly_set_pos_and_size(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slide_layouts[0].placeholders[1]


@given("a layout placeholder having no direct position or size settings")
def given_layout_placeholder_with_no_direct_pos_or_size_settings(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slide_layouts[0].placeholders[0]


@given("a master placeholder")
def given_a_master_placeholder(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.master_placeholder = prs.slide_master.placeholders[1]


@given("a notes slide placeholder having directly set position and size")
def given_notes_slide_placeholder_having_directly_set_pos_and_size(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slides[1].notes_slide.placeholders[1]


@given("a notes slide placeholder having no direct position or size settings")
def given_notes_slide_placeholder_having_no_direct_pos_or_size(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slides[0].notes_slide.placeholders[1]


@given("a slide placeholder having directly set position and size")
def given_slide_placeholder_with_directly_set_pos_and_size(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slides[0].placeholders[10]


@given("a slide placeholder having no direct position or size settings")
def given_slide_placeholder_with_no_direct_pos_or_size_settings(context):
    prs = Presentation(test_pptx("ph-inherit-props"))
    context.placeholder = prs.slides[0].placeholders[0]


@given("a slide with an unpopulated {type_} placeholder")
def given_a_slide_with_an_unpopulated_type_placeholder(context, type_):
    slide_idx = [
        "title",
        "content",
        "text",
        "chart",
        "table",
        "smart art",
        "media",
        "clip art",
        "picture",
    ].index(type_)
    prs = Presentation(test_pptx("ph-unpopulated-placeholders"))
    context.shape = prs.slides[slide_idx].shapes[0]


@given("a slide with a {type_} placeholder populated with {content}")
def given_a_slide_with_a_type_ph_with_content(context, type_, content):
    slide_idx = [
        "picture",
        "clip art",
        "table",
        "chart",
        "title",
        "content",
        "text",
        "smart art",
        "media",
    ].index(type_)
    prs = Presentation(test_pptx("ph-populated-placeholders"))
    context.shape = prs.slides[slide_idx].shapes[0]


@given("an unpopulated {placeholder_type} placeholder shape")
def given_an_unpopulated_placeholder_shape(context, placeholder_type):
    slide_idx = [
        "title",
        "content",
        "text",
        "chart",
        "table",
        "smart art",
        "media",
        "clip art",
        "picture",
    ].index(placeholder_type)
    prs = Presentation(test_pptx("ph-unpopulated-placeholders"))
    context.shape = prs.slides[slide_idx].shapes[0]


# when ====================================================


@when("I call placeholder.insert_chart(XL_CHART_TYPE.PIE, chart_data)")
def when_I_call_placeholder_insert_chart(context):
    chart_data = CategoryChartData()
    chart_data.categories = ["Yes", "No"]
    chart_data.add_series("Series 1", (42, 24))
    placeholder = context.shape
    context.placeholder = placeholder.insert_chart(XL_CHART_TYPE.PIE, chart_data)


@when("I call placeholder.insert_picture('{filename}')")
def when_I_call_placeholder_insert_picture(context, filename):
    placeholder = context.shape
    path = test_file(filename)
    with open(path, "rb") as f:
        context.image_sha1 = hashlib.sha1(f.read()).hexdigest()
    context.placeholder = placeholder.insert_picture(path)


@when("I call placeholder.insert_table(rows=2, cols=3)")
def when_I_call_placeholder_insert_table(context):
    placeholder = context.shape
    context.placeholder = placeholder.insert_table(2, 3)


@when("I indent the first paragraph")
def when_I_indent_the_first_paragraph(context):
    context.body.text_frame.paragraphs[0].level = 1


@when("I set the title text of the slide")
def step_when_set_slide_title_text(context):
    context.slide.shapes.title.text = test_text


# then ====================================================


@then("I can get the placeholder dimensions")
def then_I_can_get_the_placeholder_dimensions(context):
    placeholder = context.master_placeholder
    assert placeholder.width == 6923112, "got %d" % placeholder.width
    assert placeholder.height == 3484984, "got %d" % placeholder.height


@then("I can get the placeholder position")
def then_I_can_get_the_placeholder_position(context):
    placeholder = context.master_placeholder
    assert placeholder.left == 1110444, "got %d" % placeholder.left
    assert placeholder.top == 1686508, "got %d" % placeholder.top


@then("I get the direct settings when I query position and size")
def then_I_get_direct_settings_when_query_pos_and_size(context):
    placeholder = context.placeholder
    assert placeholder.left == 468312, "got %s" % placeholder.left
    assert placeholder.top == 1700212, "got %s" % placeholder.top
    assert placeholder.width == 8208143, "got %s" % placeholder.width
    assert placeholder.height == 4537099, "got %s" % placeholder.height


@then("I get inherited settings when I query position and size")
def then_I_get_inherited_settings_when_I_query_position_and_size(context):
    placeholder = context.placeholder
    assert placeholder.left == 457200, "got %s" % placeholder.left
    assert placeholder.top == 274638, "got %s" % placeholder.top
    assert placeholder.width == 8229600, "got %s" % placeholder.width
    assert placeholder.height == 1143000, "got %s" % placeholder.height


@then("placeholder_format.idx is {value}")
def then_placeholder_format_idx_is_value(context, value):
    expected_value = int(value)
    placeholder_format = context.shape.placeholder_format
    assert placeholder_format.idx == expected_value


@then("placeholder_format.type is {value}")
def then_placeholder_format_type_is_value(context, value):
    expected_value = getattr(PP_PLACEHOLDER, value.split(".")[1])
    placeholder_format = context.shape.placeholder_format
    assert placeholder_format.type == expected_value


@then("shape.placeholder_format is its _PlaceholderFormat object")
def then_shape_placeholder_format_is_its_PlaceholderFormat_object(context):
    shape = context.shape
    placeholder_format = shape.placeholder_format
    assert isinstance(placeholder_format, _PlaceholderFormat)
    assert placeholder_format.element is shape.element.ph


@then("shape.shape_type is MSO_SHAPE_TYPE.PLACEHOLDER")
def then_shape_shape_type_is_PLACEHOLDER(context):
    shape = context.shape
    assert shape.shape_type == MSO_SHAPE_TYPE.PLACEHOLDER


@then("slide.shapes[0] is a {cls} proxy object for that placeholder")
def then_slide_shapes_0_is_a_cls_proxy_for_that_placeholder(context, cls):
    placeholder = context.shape
    clsname = placeholder.__class__.__name__
    assert clsname == cls, "got %s" % clsname


@then("the chart is a pie chart")
def then_the_chart_is_a_pie_chart(context):
    chart = context.chart
    assert chart.chart_type == XL_CHART_TYPE.PIE


@then("the return value is a Placeholder{type} object")
def then_the_return_value_is_a_PlaceholderType_object(context, type):
    expected_type_name = "Placeholder%s" % type
    placeholder_type_name = context.placeholder.__class__.__name__
    assert placeholder_type_name == expected_type_name


@then("the paragraph is indented")
def then_the_paragraph_is_indented(context):
    prs = Presentation(saved_pptx_path)
    p = prs.slides[2].shapes.placeholders[10].text_frame.paragraphs[0]
    assert p.level == 1


@then("the placeholder contains the chart")
def then_the_placeholder_contains_the_chart(context):
    placeholder_graphic_frame = context.placeholder
    assert placeholder_graphic_frame.has_chart
    context.chart = placeholder_graphic_frame.chart


@then("the placeholder contains the image")
def then_the_placeholder_contains_the_image(context):
    placeholder_picture = context.placeholder
    assert placeholder_picture.image.sha1 == context.image_sha1


@then("the placeholder contains the table")
def then_the_placeholder_contains_the_table(context):
    placeholder_graphic_frame = context.placeholder
    assert placeholder_graphic_frame.has_table
    context.table_ = placeholder_graphic_frame.table


@then("the placeholder's position and size are inherited from its layout")
def then_the_placeholders_position_and_size_are_inherited(context):
    placeholder = context.shape
    expected_values = (
        ("left", 2743200),
        ("top", 2057400),
        ("width", 3657600),
        ("height", 2743200),
    )
    for prop_name, expected_value in expected_values:
        value = getattr(placeholder, prop_name)
        assert value == expected_value, "got %s" % value


@then("the {sides} crop is {value}")
def then_the_sides_crop_is_value(context, sides, value):
    side_prop_names = {
        "top and bottom": ("crop_top", "crop_bottom"),
        "left and right": ("crop_left", "crop_right"),
    }[sides]
    expected_value = float(value)
    placeholder_picture = context.placeholder
    for prop_name in side_prop_names:
        value = getattr(placeholder_picture, prop_name)
        difference = abs(expected_value - value)
        assert difference < 0.000002, "got %s for %s" % (value, prop_name)


@then("the table has 2 rows and 3 columns")
def then_the_table_has_2_rows_and_3_columns(context):
    table = context.table_
    assert len(table.rows) == 2
    assert len(table.columns) == 3


@then("the text appears in the title placeholder")
def step_then_text_appears_in_title_placeholder(context):
    prs = Presentation(saved_pptx_path)
    title_shape = prs.slides[0].shapes.title
    title_text = title_shape.text_frame.paragraphs[0].runs[0].text
    assert title_text == test_text



# encoding: utf-8

"""DrawingML objects related to fill."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.compat import Sequence
from pptx.dml.color import ColorFormat
from pptx.enum.dml import MSO_FILL
from pptx.oxml.dml.fill import (
    CT_BlipFillProperties,
    CT_GradientFillProperties,
    CT_GroupFillProperties,
    CT_NoFillProperties,
    CT_PatternFillProperties,
    CT_SolidColorFillProperties,
)
from pptx.shared import ElementProxy
from pptx.util import lazyproperty


class FillFormat(object):
    """
    Provides access to the current fill properties object and provides
    methods to change the fill type.
    """

    def __init__(self, eg_fill_properties_parent, fill_obj):
        super(FillFormat, self).__init__()
        self._xPr = eg_fill_properties_parent
        self._fill = fill_obj

    @classmethod
    def from_fill_parent(cls, eg_fillProperties_parent):
        """
        Return a |FillFormat| instance initialized to the settings contained
        in *eg_fillProperties_parent*, which must be an element having
        EG_FillProperties in its child element sequence in the XML schema.
        """
        fill_elm = eg_fillProperties_parent.eg_fillProperties
        fill = _Fill(fill_elm)
        fill_format = cls(eg_fillProperties_parent, fill)
        return fill_format

    @property
    def back_color(self):
        """Return a |ColorFormat| object representing background color.

        This property is only applicable to pattern fills and lines.
        """
        return self._fill.back_color

    def background(self):
        """
        Sets the fill type to noFill, i.e. transparent.
        """
        noFill = self._xPr.get_or_change_to_noFill()
        self._fill = _NoFill(noFill)

    @property
    def fore_color(self):
        """
        Return a |ColorFormat| instance representing the foreground color of
        this fill.
        """
        return self._fill.fore_color

    def gradient(self):
        """Sets the fill type to gradient.

        If the fill is not already a gradient, a default gradient is added.
        The default gradient corresponds to the default in the built-in
        PowerPoint "White" template. This gradient is linear at angle
        90-degrees (upward), with two stops. The first stop is Accent-1 with
        tint 100%, shade 100%, and satMod 130%. The second stop is Accent-1
        with tint 50%, shade 100%, and satMod 350%.
        """
        gradFill = self._xPr.get_or_change_to_gradFill()
        self._fill = _GradFill(gradFill)

    @property
    def gradient_angle(self):
        """Angle in float degrees of line of a linear gradient.

        Read/Write. May be |None|, indicating the angle should be inherited
        from the style hierarchy. An angle of 0.0 corresponds to
        a left-to-right gradient. Increasing angles represent
        counter-clockwise rotation of the line, for example 90.0 represents
        a bottom-to-top gradient. Raises |TypeError| when the fill type is
        not MSO_FILL_TYPE.GRADIENT. Raises |ValueError| for a non-linear
        gradient (e.g. a radial gradient).
        """
        if self.type != MSO_FILL.GRADIENT:
            raise TypeError("Fill is not of type MSO_FILL_TYPE.GRADIENT")
        return self._fill.gradient_angle

    @gradient_angle.setter
    def gradient_angle(self, value):
        if self.type != MSO_FILL.GRADIENT:
            raise TypeError("Fill is not of type MSO_FILL_TYPE.GRADIENT")
        self._fill.gradient_angle = value

    @property
    def gradient_stops(self):
        """|GradientStops| object providing access to stops of this gradient.

        Raises |TypeError| when fill is not gradient (call `fill.gradient()`
        first). Each stop represents a color between which the gradient
        smoothly transitions.
        """
        if self.type != MSO_FILL.GRADIENT:
            raise TypeError("Fill is not of type MSO_FILL_TYPE.GRADIENT")
        return self._fill.gradient_stops

    @property
    def pattern(self):
        """Return member of :ref:`MsoPatternType` indicating fill pattern.

        Raises |TypeError| when fill is not patterned (call
        `fill.patterned()` first). Returns |None| if no pattern has been set;
        PowerPoint may display the default `PERCENT_5` pattern in this case.
        Assigning |None| will remove any explicit pattern setting, although
        relying on the default behavior is discouraged and may produce
        rendering differences across client applications.
        """
        return self._fill.pattern

    @pattern.setter
    def pattern(self, pattern_type):
        self._fill.pattern = pattern_type

    def patterned(self):
        """Selects the pattern fill type.

        Note that calling this method does not by itself set a foreground or
        background color of the pattern. Rather it enables subsequent
        assignments to properties like fore_color to set the pattern and
        colors.
        """
        pattFill = self._xPr.get_or_change_to_pattFill()
        self._fill = _PattFill(pattFill)

    def solid(self):
        """
        Sets the fill type to solid, i.e. a solid color. Note that calling
        this method does not set a color or by itself cause the shape to
        appear with a solid color fill; rather it enables subsequent
        assignments to properties like fore_color to set the color.
        """
        solidFill = self._xPr.get_or_change_to_solidFill()
        self._fill = _SolidFill(solidFill)

    @property
    def type(self):
        """
        Return a value from the :ref:`MsoFillType` enumeration corresponding
        to the type of this fill.
        """
        return self._fill.type


class _Fill(object):
    """
    Object factory for fill object of class matching fill element, such as
    _SolidFill for ``<a:solidFill>``; also serves as the base class for all
    fill classes
    """

    def __new__(cls, xFill):
        if xFill is None:
            fill_cls = _NoneFill
        elif isinstance(xFill, CT_BlipFillProperties):
            fill_cls = _BlipFill
        elif isinstance(xFill, CT_GradientFillProperties):
            fill_cls = _GradFill
        elif isinstance(xFill, CT_GroupFillProperties):
            fill_cls = _GrpFill
        elif isinstance(xFill, CT_NoFillProperties):
            fill_cls = _NoFill
        elif isinstance(xFill, CT_PatternFillProperties):
            fill_cls = _PattFill
        elif isinstance(xFill, CT_SolidColorFillProperties):
            fill_cls = _SolidFill
        else:
            fill_cls = _Fill
        return super(_Fill, cls).__new__(fill_cls)

    @property
    def back_color(self):
        """Raise TypeError for types that do not override this property."""
        tmpl = "fill type %s has no background color, call .patterned() first"
        raise TypeError(tmpl % self.__class__.__name__)

    @property
    def fore_color(self):
        """Raise TypeError for types that do not override this property."""
        tmpl = (
            "fill type %s has no foreground color, call .solid() or .pattern"
            "ed() first"
        )
        raise TypeError(tmpl % self.__class__.__name__)

    @property
    def pattern(self):
        """Raise TypeError for fills that do not override this property."""
        tmpl = "fill type %s has no pattern, call .patterned() first"
        raise TypeError(tmpl % self.__class__.__name__)

    @property
    def type(self):  # pragma: no cover
        tmpl = ".type property must be implemented on %s"
        raise NotImplementedError(tmpl % self.__class__.__name__)


class _BlipFill(_Fill):
    @property
    def type(self):
        return MSO_FILL.PICTURE


class _GradFill(_Fill):
    """Proxies an `a:gradFill` element."""

    def __init__(self, gradFill):
        self._element = self._gradFill = gradFill

    @property
    def gradient_angle(self):
        """Angle in float degrees of line of a linear gradient.

        Read/Write. May be |None|, indicating the angle is inherited from the
        style hierarchy. An angle of 0.0 corresponds to a left-to-right
        gradient. Increasing angles represent clockwise rotation of the line,
        for example 90.0 represents a top-to-bottom gradient. Raises
        |TypeError| when the fill type is not MSO_FILL_TYPE.GRADIENT. Raises
        |ValueError| for a non-linear gradient (e.g. a radial gradient).
        """
        # ---case 1: gradient path is explicit, but not linear---
        path = self._gradFill.path
        if path is not None:
            raise ValueError("not a linear gradient")

        # ---case 2: gradient path is inherited (no a:lin OR a:path)---
        lin = self._gradFill.lin
        if lin is None:
            return None

        # ---case 3: gradient path is explicitly linear---
        # angle is stored in XML as a clockwise angle, whereas the UI
        # reports it as counter-clockwise from horizontal-pointing-right.
        # Since the UI is consistent with trigonometry conventions, we
        # respect that in the API.
        clockwise_angle = lin.ang
        counter_clockwise_angle = (
            0.0 if clockwise_angle == 0.0 else (360.0 - clockwise_angle)
        )
        return counter_clockwise_angle

    @gradient_angle.setter
    def gradient_angle(self, value):
        lin = self._gradFill.lin
        if lin is None:
            raise ValueError("not a linear gradient")
        lin.ang = 360.0 - value

    @lazyproperty
    def gradient_stops(self):
        """|_GradientStops| object providing access to gradient colors.

        Each stop represents a color between which the gradient smoothly
        transitions.
        """
        return _GradientStops(self._gradFill.get_or_add_gsLst())

    @property
    def type(self):
        return MSO_FILL.GRADIENT


class _GrpFill(_Fill):
    @property
    def type(self):
        return MSO_FILL.GROUP


class _NoFill(_Fill):
    @property
    def type(self):
        return MSO_FILL.BACKGROUND


class _NoneFill(_Fill):
    @property
    def type(self):
        return None


class _PattFill(_Fill):
    """Provides access to patterned fill properties."""

    def __init__(self, pattFill):
        super(_PattFill, self).__init__()
        self._element = self._pattFill = pattFill

    @lazyproperty
    def back_color(self):
        """Return |ColorFormat| object that controls background color."""
        bgClr = self._pattFill.get_or_add_bgClr()
        return ColorFormat.from_colorchoice_parent(bgClr)

    @lazyproperty
    def fore_color(self):
        """Return |ColorFormat| object that controls foreground color."""
        fgClr = self._pattFill.get_or_add_fgClr()
        return ColorFormat.from_colorchoice_parent(fgClr)

    @property
    def pattern(self):
        """Return member of :ref:`MsoPatternType` indicating fill pattern.

        Returns |None| if no pattern has been set; PowerPoint may display the
        default `PERCENT_5` pattern in this case. Assigning |None| will
        remove any explicit pattern setting.
        """
        return self._pattFill.prst

    @pattern.setter
    def pattern(self, pattern_type):
        self._pattFill.prst = pattern_type

    @property
    def type(self):
        return MSO_FILL.PATTERNED


class _SolidFill(_Fill):
    """Provides access to fill properties such as color for solid fills."""

    def __init__(self, solidFill):
        super(_SolidFill, self).__init__()
        self._solidFill = solidFill

    @lazyproperty
    def fore_color(self):
        """Return |ColorFormat| object controlling fill color."""
        return ColorFormat.from_colorchoice_parent(self._solidFill)

    @property
    def type(self):
        return MSO_FILL.SOLID


class _GradientStops(Sequence):
    """Collection of |GradientStop| objects defining gradient colors.

    A gradient must have a minimum of two stops, but can have as many more
    than that as required to achieve the desired effect (three is perhaps
    most common). Stops are sequenced in the order they are transitioned
    through.
    """

    def __init__(self, gsLst):
        self._gsLst = gsLst

    def __getitem__(self, idx):
        return _GradientStop(self._gsLst[idx])

    def __len__(self):
        return len(self._gsLst)


class _GradientStop(ElementProxy):
    """A single gradient stop.

    A gradient stop defines a color and a position.
    """

    __slots__ = ("_gs", "_color")

    def __init__(self, gs):
        super(_GradientStop, self).__init__(gs)
        self._gs = gs

    @lazyproperty
    def color(self):
        """Return |ColorFormat| object controlling stop color."""
        return ColorFormat.from_colorchoice_parent(self._gs)

    @property
    def position(self):
        """Location of stop in gradient path as float between 0.0 and 1.0.

        The value represents a percentage, where 0.0 (0%) represents the
        start of the path and 1.0 (100%) represents the end of the path. For
        a linear gradient, these would represent opposing extents of the
        filled area.
        """
        return self._gs.pos

    @position.setter
    def position(self, value):
        self._gs.pos = float(value)



# encoding: utf-8

"""
Test suite for pptx.chart.data module
"""

from __future__ import absolute_import, print_function, unicode_literals

from datetime import date, datetime

import pytest

from pptx.chart.data import (
    _BaseChartData,
    _BaseDataPoint,
    _BaseSeriesData,
    BubbleChartData,
    BubbleDataPoint,
    BubbleSeriesData,
    Categories,
    Category,
    CategoryChartData,
    CategoryDataPoint,
    CategorySeriesData,
    ChartData,
    XyChartData,
    XyDataPoint,
    XySeriesData,
)
from pptx.chart.xlsx import CategoryWorkbookWriter
from pptx.enum.base import EnumValue

from ..unitutil.mock import call, class_mock, instance_mock, property_mock


class DescribeChartData(object):
    def it_is_a_CategoryChartData_object(self):
        assert isinstance(ChartData(), CategoryChartData)


class Describe_BaseChartData(object):
    def it_can_generate_chart_part_XML_for_its_data(self, xml_bytes_fixture):
        chart_data, chart_type_, ChartXmlWriter_, expected_bytes = xml_bytes_fixture
        xml_bytes = chart_data.xml_bytes(chart_type_)

        ChartXmlWriter_.assert_called_once_with(chart_type_, chart_data)
        assert xml_bytes == expected_bytes

    def it_knows_its_number_format(self, number_format_fixture):
        chart_data, expected_value = number_format_fixture
        assert chart_data.number_format == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[(None, "General"), (42, 42)])
    def number_format_fixture(self, request):
        number_format, expected_value = request.param
        argv = [] if number_format is None else [number_format]
        chart_data = _BaseChartData(*argv)
        return chart_data, expected_value

    @pytest.fixture
    def xml_bytes_fixture(self, chart_type_, ChartXmlWriter_):
        chart_data = _BaseChartData()
        expected_bytes = "ƒøØßår".encode("utf-8")
        return chart_data, chart_type_, ChartXmlWriter_, expected_bytes

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ChartXmlWriter_(self, request):
        ChartXmlWriter_ = class_mock(request, "pptx.chart.data.ChartXmlWriter")
        ChartXmlWriter_.return_value.xml = "ƒøØßår"
        return ChartXmlWriter_

    @pytest.fixture
    def chart_type_(self, request):
        return instance_mock(request, EnumValue)


class Describe_BaseSeriesData(object):
    def it_knows_its_name(self, name_fixture):
        name_arg, expected_value = name_fixture
        series_data = _BaseSeriesData(None, name_arg, None)

        name = series_data.name

        assert name == expected_value

    def it_knows_its_number_format(self, number_format_fixture):
        series_data, expected_value = number_format_fixture
        assert series_data.number_format == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[("Tincture of Foo", "Tincture of Foo"), (None, "")])
    def name_fixture(self, request):
        name, expected_value = request.param
        return name, expected_value

    @pytest.fixture(params=[("General", 42, 42), ("General", None, "General")])
    def number_format_fixture(self, request, chart_data_):
        parent_number_format, number_format, expected_value = request.param
        chart_data_.number_format = parent_number_format
        series_data = _BaseSeriesData(chart_data_, "Foobar", number_format)
        return series_data, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, _BaseChartData)


class Describe_BaseDataPoint(object):
    def it_knows_its_number_format(self, number_format_fixture):
        data_point, expected_value = number_format_fixture
        assert data_point.number_format == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[(42, 24, 24), (42, None, 42)])
    def number_format_fixture(self, request, series_data_):
        parent_number_format, number_format, expected_value = request.param
        series_data_.number_format = parent_number_format
        data_point = _BaseDataPoint(series_data_, number_format)
        return data_point, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, _BaseSeriesData)


class DescribeCategoryChartData(object):
    def it_is_a__BaseChartData_object(self):
        assert isinstance(CategoryChartData(), _BaseChartData)

    def it_knows_the_categories_range_ref(self, categories_ref_fixture):
        chart_data, expected_value = categories_ref_fixture
        assert chart_data.categories_ref == expected_value

    def it_knows_the_values_range_ref_for_a_series(self, values_ref_fixture):
        chart_data, workbook_writer_, series_, values_ref_ = values_ref_fixture
        values_ref = chart_data.values_ref(series_)
        workbook_writer_.values_ref.assert_called_once_with(series_)
        assert values_ref is values_ref_

    def it_provides_access_to_its_categories(self, categories_fixture):
        chart_data, Categories_, categories_ = categories_fixture
        categories = chart_data.categories
        Categories_.assert_called_once_with()
        assert categories is categories_

    def it_can_add_a_category(self, add_cat_fixture):
        chart_data, name, categories_, category_ = add_cat_fixture
        category = chart_data.add_category(name)
        categories_.add_category.assert_called_once_with(name)
        assert category is category_

    def it_can_add_a_series(self, add_ser_fixture):
        chart_data, name, values, number_format = add_ser_fixture[:4]
        CategorySeriesData_, calls, series_ = add_ser_fixture[4:]
        series = chart_data.add_series(name, values, number_format)
        CategorySeriesData_.assert_called_once_with(chart_data, name, number_format)
        assert chart_data[-1] is series
        assert series.add_data_point.call_args_list == calls
        assert series is series_

    def it_can_set_its_categories(self, categories_set_fixture):
        chart_data, names, Categories_, categories_, calls = categories_set_fixture
        chart_data.categories = names
        Categories_.assert_called_once_with()
        assert categories_.add_category.call_args_list == calls
        assert chart_data._categories is categories_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_cat_fixture(self, categories_prop_, categories_, category_):
        chart_data = CategoryChartData()
        name = "foobar"
        categories_.add_category.return_value = category_
        return chart_data, name, categories_, category_

    @pytest.fixture
    def add_ser_fixture(self, CategorySeriesData_, series_):
        chart_data = CategoryChartData()
        name, values, number_format = "foobar", iter((1, 2, 3)), "0.0"
        calls = [call(1), call(2), call(3)]
        return (
            chart_data,
            name,
            values,
            number_format,
            CategorySeriesData_,
            calls,
            series_,
        )

    @pytest.fixture
    def categories_fixture(self, Categories_, categories_):
        chart_data = CategoryChartData()
        return chart_data, Categories_, categories_

    @pytest.fixture
    def categories_ref_fixture(self, _workbook_writer_prop_, workbook_writer_):
        chart_data = CategoryChartData()
        expected_value = categories_ref = "Sheet42!$G$24"
        workbook_writer_.categories_ref = categories_ref
        return chart_data, expected_value

    @pytest.fixture
    def categories_set_fixture(self, Categories_, categories_):
        chart_data = CategoryChartData()
        names = iter(("a", "b", "c"))
        calls = [call("a"), call("b"), call("c")]
        return chart_data, names, Categories_, categories_, calls

    @pytest.fixture
    def values_ref_fixture(self, _workbook_writer_prop_, workbook_writer_, series_):
        chart_data = CategoryChartData()
        values_ref_ = "Sheet1!$V$24"
        workbook_writer_.values_ref.return_value = values_ref_
        return chart_data, workbook_writer_, series_, values_ref_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Categories_(self, request, categories_):
        return class_mock(
            request, "pptx.chart.data.Categories", return_value=categories_
        )

    @pytest.fixture
    def CategorySeriesData_(self, request, series_):
        return class_mock(
            request, "pptx.chart.data.CategorySeriesData", return_value=series_
        )

    @pytest.fixture
    def categories_(self, request):
        return instance_mock(request, Categories)

    @pytest.fixture
    def categories_prop_(self, request, categories_):
        return property_mock(
            request, CategoryChartData, "categories", return_value=categories_
        )

    @pytest.fixture
    def category_(self, request):
        return instance_mock(request, Category)

    @pytest.fixture
    def series_(self, request):
        return instance_mock(request, CategorySeriesData)

    @pytest.fixture
    def workbook_writer_(self, request):
        return instance_mock(request, CategoryWorkbookWriter)

    @pytest.fixture
    def _workbook_writer_prop_(self, request, workbook_writer_):
        return property_mock(
            request,
            CategoryChartData,
            "_workbook_writer",
            return_value=workbook_writer_,
        )


class DescribeCategories(object):
    def it_knows_when_its_categories_are_numeric(self, are_numeric_fixture):
        categories, expected_value = are_numeric_fixture
        assert categories.are_numeric == expected_value

    def it_knows_when_its_categories_are_dates(self, are_dates_fixture):
        categories, expected_value = are_dates_fixture
        assert categories.are_dates == expected_value

    def it_knows_the_category_hierarchy_depth(self, depth_fixture):
        categories, expected_value = depth_fixture
        assert categories.depth == expected_value

    def it_knows_the_idx_of_a_category(self, index_fixture):
        categories, category_, expected_value = index_fixture
        assert categories.index(category_) == expected_value

    def it_knows_its_leaf_category_count(self, leaf_fixture):
        categories, expected_value = leaf_fixture
        assert categories.leaf_count == expected_value

    def it_knows_its_levels(self, levels_fixture):
        categories, expected_value = levels_fixture
        assert list(categories.levels) == expected_value

    def it_knows_its_number_format(self, number_format_get_fixture):
        categories, expected_value = number_format_get_fixture
        assert categories.number_format == expected_value

    def it_can_change_its_number_format(self, number_format_set_fixture):
        categories, new_value, expected_value = number_format_set_fixture
        categories.number_format = new_value
        assert categories.number_format == expected_value

    def it_raises_on_category_depth_not_uniform(self, depth_raises_fixture):
        categories = depth_raises_fixture
        with pytest.raises(ValueError):
            categories.depth

    def it_can_add_a_category(self, add_fixture):
        categories, name, Category_, category_ = add_fixture
        category = categories.add_category(name)
        Category_.assert_called_once_with(name, categories)
        assert categories._categories[-1] is category
        assert category is category_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self, Category_, category_):
        categories = Categories()
        name = "foobar"
        return categories, name, Category_, category_

    @pytest.fixture(
        params=[
            ((), False),
            (("foo", "bar"), False),
            ((1, 2), False),
            ((1.2, 2.3), False),
            ((date(2016, 12, 21),), True),
        ]
    )
    def are_dates_fixture(self, request):
        labels, expected_value = request.param
        categories = Categories()
        for label in labels:
            categories.add_category(label)
        return categories, expected_value

    @pytest.fixture(
        params=[
            ((), False),
            (("foo", "bar"), False),
            ((1, 2), True),
            ((1.2, 2.3), True),
            ((date(2016, 12, 21),), True),
        ]
    )
    def are_numeric_fixture(self, request):
        labels, expected_value = request.param
        categories = Categories()
        for label in labels:
            categories.add_category(label)
        return categories, expected_value

    @pytest.fixture(
        params=[((), 0), ((1,), 1), ((3,), 3), ((1, 1, 1), 1), ((3, 3, 3), 3)]
    )
    def depth_fixture(self, request):
        depths, expected_value = request.param
        categories = Categories()
        for depth in depths:
            categories._categories.append(instance_mock(request, Category, depth=depth))
        return categories, expected_value

    @pytest.fixture
    def depth_raises_fixture(self, request):
        categories = Categories()
        for depth in (1, 2, 1):
            categories._categories.append(instance_mock(request, Category, depth=depth))
        return categories

    @pytest.fixture
    def index_fixture(self, request):
        categories = Categories()
        categories_ = [
            instance_mock(request, Category, leaf_count=3),
            instance_mock(request, Category, leaf_count=6),
            instance_mock(request, Category, leaf_count=9),
        ]
        category_ = categories_[1]
        expected_value = 3
        categories._categories = categories_
        return categories, category_, expected_value

    @pytest.fixture(params=[((), 0), ((1,), 1), ((1, 2, 3), 6)])
    def leaf_fixture(self, request):
        leaf_counts, expected_value = request.param
        categories = Categories()
        for leaf_count in leaf_counts:
            categories._categories.append(
                instance_mock(request, Category, leaf_count=leaf_count)
            )
        return categories, expected_value

    @pytest.fixture(
        params=[
            ([(0, "a", ()), (1, "b", ())], [[(0, "a"), (1, "b")]]),
            (
                [
                    (0, "WEST", ((0, "CA", ()), (1, "LA", ()))),
                    (2, "EAST", ((2, "NY", ()), (3, "NJ", ()))),
                ],
                [
                    [(0, "CA"), (1, "LA"), (2, "NY"), (3, "NJ")],
                    [(0, "WEST"), (2, "EAST")],
                ],
            ),
        ]
    )
    def levels_fixture(self, request):
        cat_data, expected_value = request.param
        categories = Categories()

        def iter_cats(cat_tree):
            for idx, cat_label, sub_cats in cat_tree:
                category_ = instance_mock(request, Category, idx=idx)
                category_.label = cat_label
                category_.sub_categories = list(iter_cats(sub_cats))
                yield category_

        categories._categories = list(iter_cats(cat_data))

        return categories, expected_value

    @pytest.fixture(
        params=[
            (None, None, None, "General"),
            (None, "foo", None, "General"),
            (None, "foo", "bar", "General"),
            (None, date(2016, 12, 22), None, r"yyyy\-mm\-dd"),
            (None, date(2016, 12, 22), "foo", "General"),
            ("#0", 42.24, None, "#0"),
        ]
    )
    def number_format_get_fixture(self, request):
        number_format, cat, subcat, expected_value = request.param
        categories = Categories()
        if cat is not None:
            category = categories.add_category(cat)
            if subcat is not None:
                category.add_sub_category(subcat)
        if number_format is not None:
            categories._number_format = number_format
        return categories, expected_value

    @pytest.fixture(params=[("0.0", "0.0"), (None, "General")])
    def number_format_set_fixture(self, request):
        new_value, expected_value = request.param
        categories = Categories()
        return categories, new_value, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Category_(self, request, category_):
        return class_mock(request, "pptx.chart.data.Category", return_value=category_)

    @pytest.fixture
    def category_(self, request):
        return instance_mock(request, Category)


class DescribeCategory(object):
    def it_knows_its_depth(self, depth_fixture):
        category, expected_value = depth_fixture
        assert category.depth == expected_value

    def it_knows_its_idx(self, idx_fixture):
        category, parent_, idx_ = idx_fixture
        idx = category.idx
        parent_.index.assert_called_once_with(category)
        assert idx == idx_

    def it_knows_the_index_of_a_sub_category(self, index_fixture):
        category, sub_category_, expected_value = index_fixture
        index = category.index(sub_category_)
        category._parent.index.assert_called_once_with(category)
        assert index == expected_value

    def it_knows_its_leaf_category_count(self, leaf_fixture):
        category, expected_value = leaf_fixture
        assert category.leaf_count == expected_value

    def it_raises_on_depth_not_uniform(self, depth_raises_fixture):
        category = depth_raises_fixture
        with pytest.raises(ValueError):
            category.depth

    def it_knows_its_label(self, label_fixture):
        label_arg, expected_value = label_fixture
        category = Category(label_arg, None)

        label = category.label

        assert label == expected_value

    def it_knows_its_numeric_string_value(self, num_str_fixture):
        category, date_1904, expected_value = num_str_fixture
        assert category.numeric_str_val(date_1904) == expected_value

    def it_provides_access_to_its_sub_categories(self, subs_fixture):
        category, sub_categories_ = subs_fixture
        assert category.sub_categories is sub_categories_

    def it_can_add_a_sub_category(self, add_sub_fixture):
        category, name, Category_, sub_category_ = add_sub_fixture
        sub_category = category.add_sub_category(name)
        Category_.assert_called_once_with(name, category)
        assert category._sub_categories[-1] is sub_category
        assert sub_category is sub_category_

    def it_calculates_an_excel_date_number_to_help(self, excel_date_fixture):
        category, date_1904, expected_value = excel_date_fixture
        date_number = category._excel_date_number(date_1904)
        assert date_number == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_sub_fixture(self, request, category_):
        category = Category(None, None)
        name = "foobar"
        Category_ = class_mock(
            request, "pptx.chart.data.Category", return_value=category_
        )
        return category, name, Category_, category_

    @pytest.fixture(params=[((), 1), ((1,), 2), ((1, 1, 1), 2), ((2, 2, 2), 3)])
    def depth_fixture(self, request):
        depths, expected_value = request.param
        category = Category(None, None)
        for depth in depths:
            category._sub_categories.append(
                instance_mock(request, Category, depth=depth)
            )
        return category, expected_value

    @pytest.fixture
    def depth_raises_fixture(self, request):
        category = Category(None, None)
        for depth in (1, 2, 1):
            category._sub_categories.append(
                instance_mock(request, Category, depth=depth)
            )
        return category

    @pytest.fixture(
        params=[
            (date(2016, 12, 22), False, 42726),
            (date(1999, 12, 31), True, 35063),
            (datetime(1900, 2, 28), False, 59),
            (datetime(1990, 9, 1), True, 31655),
        ]
    )
    def excel_date_fixture(self, request):
        label, date_1904, expected_value = request.param
        category = Category(label, None)
        return category, date_1904, expected_value

    @pytest.fixture(params=[Categories, Category])
    def idx_fixture(self, request):
        parent_cls = request.param
        parent_ = instance_mock(request, parent_cls)
        category = Category(None, parent_)
        parent_.index.return_value = idx_ = 42
        return category, parent_, idx_

    @pytest.fixture
    def index_fixture(self, request, categories_):
        category = Category(None, categories_)
        sub_categories_ = [
            instance_mock(request, Category, leaf_count=2),
            instance_mock(request, Category, leaf_count=4),
            instance_mock(request, Category, leaf_count=6),
        ]
        sub_category_ = sub_categories_[1]
        expected_value = 6
        categories_.index.return_value = 4
        category._sub_categories = sub_categories_
        return category, sub_category_, expected_value

    @pytest.fixture(params=[("Able the Label", "Able the Label"), (None, "")])
    def label_fixture(self, request):
        label, expected_value = request.param
        return label, expected_value

    @pytest.fixture(params=[((), 1), ((1,), 1), ((1, 2, 3), 6)])
    def leaf_fixture(self, request):
        leaf_counts, expected_value = request.param
        category = Category(None, None)
        for leaf_count in leaf_counts:
            category._sub_categories.append(
                instance_mock(request, Category, leaf_count=leaf_count)
            )
        return category, expected_value

    @pytest.fixture(
        params=[
            (date(2016, 12, 23), False, "42727.0"),
            (42.24, False, "42.24"),
            (42, False, "42"),
            ("foobar", False, "foobar"),
        ]
    )
    def num_str_fixture(self, request):
        label, date_1904, expected_value = request.param
        category = Category(label, None)
        return category, date_1904, expected_value

    @pytest.fixture
    def subs_fixture(self):
        sub_categories_ = [42, 24]
        category = Category(None, None)
        category._sub_categories = sub_categories_
        return category, sub_categories_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def categories_(self, request):
        return instance_mock(request, Categories)

    @pytest.fixture
    def category_(self, request):
        return instance_mock(request, Category)


class DescribeCategorySeriesData(object):
    def it_knows_the_categories_range_ref(self, categories_ref_fixture):
        series_data, expected_value = categories_ref_fixture
        assert series_data.categories_ref == expected_value

    def it_knows_its_values(self, values_fixture):
        series_data, expected_values = values_fixture
        assert series_data.values == expected_values

    def it_knows_its_values_range_ref(self, values_ref_fixture):
        series_data, chart_data_, values_ref_ = values_ref_fixture
        values_ref = series_data.values_ref
        chart_data_.values_ref.assert_called_once_with(series_data)
        assert values_ref is values_ref_

    def it_provides_access_to_the_chart_categories(self, categories_fixture):
        series_data, categories_ = categories_fixture
        assert series_data.categories is categories_

    def it_can_add_a_data_point(self, add_fixture):
        series_data, value, number_format = add_fixture[:3]
        CategoryDataPoint_, data_point_ = add_fixture[3:]
        data_point = series_data.add_data_point(value, number_format)
        CategoryDataPoint_.assert_called_once_with(series_data, value, number_format)
        assert series_data[-1] is data_point_
        assert data_point is data_point_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self, request, CategoryDataPoint_, data_point_):
        series_data = CategorySeriesData(None, None, None)
        value, number_format = 42, "0.0"
        return (series_data, value, number_format, CategoryDataPoint_, data_point_)

    @pytest.fixture
    def categories_fixture(self, chart_data_, categories_):
        series_data = CategorySeriesData(chart_data_, None, None)
        chart_data_.categories = categories_
        return series_data, categories_

    @pytest.fixture
    def categories_ref_fixture(self, chart_data_):
        series_data = CategorySeriesData(chart_data_, None, None)
        expected_value = categories_ref = "Sheet1!$F$42"
        chart_data_.categories_ref = categories_ref
        return series_data, expected_value

    @pytest.fixture
    def values_fixture(self, request):
        series_data = CategorySeriesData(None, None, None)
        expected_values = [1, 2, 3]
        for value in expected_values:
            series_data._data_points.append(
                instance_mock(request, CategoryDataPoint, value=value)
            )
        return series_data, expected_values

    @pytest.fixture
    def values_ref_fixture(self, chart_data_):
        series_data = CategorySeriesData(chart_data_, None, None)
        values_ref_ = "Sheet1!$V$42"
        chart_data_.values_ref.return_value = values_ref_
        return series_data, chart_data_, values_ref_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def CategoryDataPoint_(self, request, data_point_):
        return class_mock(
            request, "pptx.chart.data.CategoryDataPoint", return_value=data_point_
        )

    @pytest.fixture
    def categories_(self, request):
        return instance_mock(request, Categories)

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, CategoryChartData)

    @pytest.fixture
    def data_point_(self, request):
        return instance_mock(request, CategoryDataPoint)


class DescribeBubbleChartData(object):
    def it_can_add_a_series(self, add_series_fixture):
        chart_data, name, BubbleSeriesData_, series_data_ = add_series_fixture
        series_data = chart_data.add_series(name)
        BubbleSeriesData_.assert_called_once_with(chart_data, name, None)
        assert chart_data[-1] is series_data_
        assert series_data is series_data_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_series_fixture(self, request, BubbleSeriesData_, series_data_):
        chart_data = BubbleChartData()
        name = "Series Name"
        return chart_data, name, BubbleSeriesData_, series_data_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def BubbleSeriesData_(self, request, series_data_):
        return class_mock(
            request, "pptx.chart.data.BubbleSeriesData", return_value=series_data_
        )

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, BubbleSeriesData)


class DescribeXyChartData(object):
    def it_is_a__BaseChartData_object(self):
        assert isinstance(XyChartData(), _BaseChartData)

    def it_can_add_a_series(self, add_series_fixture):
        chart_data, label, XySeriesData_, series_data_ = add_series_fixture
        series_data = chart_data.add_series(label)
        XySeriesData_.assert_called_once_with(chart_data, label, None)
        assert chart_data[-1] is series_data_
        assert series_data is series_data_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_series_fixture(self, request, XySeriesData_, series_data_):
        chart_data = XyChartData()
        label = "Series Label"
        return chart_data, label, XySeriesData_, series_data_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def XySeriesData_(self, request, series_data_):
        return class_mock(
            request, "pptx.chart.data.XySeriesData", return_value=series_data_
        )

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, XySeriesData)


class DescribeBubbleSeriesData(object):
    def it_can_add_a_data_point(self, add_data_point_fixture):
        series_data, x, y, size, BubbleDataPoint_, data_point_ = add_data_point_fixture
        data_point = series_data.add_data_point(x, y, size)
        BubbleDataPoint_.assert_called_once_with(series_data, x, y, size, None)
        assert series_data[-1] is data_point_
        assert data_point is data_point_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_data_point_fixture(self, request, BubbleDataPoint_, data_point_):
        series_data = BubbleSeriesData(None, None, None)
        x, y, size = 42, 24, 17
        return series_data, x, y, size, BubbleDataPoint_, data_point_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def BubbleDataPoint_(self, request, data_point_):
        return class_mock(
            request, "pptx.chart.data.BubbleDataPoint", return_value=data_point_
        )

    @pytest.fixture
    def data_point_(self, request):
        return instance_mock(request, BubbleDataPoint)


class DescribeXySeriesData(object):
    def it_is_a__BaseSeriesData_object(self, chart_data_):
        name, number_format = "Series 42", "#0.0"
        series_data = XySeriesData(chart_data_, name, number_format)
        assert isinstance(series_data, _BaseSeriesData)
        assert series_data._chart_data is chart_data_
        assert series_data.name == name
        assert series_data.number_format == number_format

    def it_can_add_a_data_point(self, add_data_point_fixture):
        series_data, x, y, XyDataPoint_, data_point_ = add_data_point_fixture
        data_point = series_data.add_data_point(x, y)
        XyDataPoint_.assert_called_once_with(series_data, x, y, None)
        assert series_data[-1] is data_point_
        assert data_point is data_point_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_data_point_fixture(self, request, XyDataPoint_, data_point_):
        series_data = XySeriesData(None, None, None)
        x, y = 42, 24
        return series_data, x, y, XyDataPoint_, data_point_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, _BaseChartData)

    @pytest.fixture
    def data_point_(self, request):
        return instance_mock(request, XyDataPoint)

    @pytest.fixture
    def XyDataPoint_(self, request, data_point_):
        return class_mock(
            request, "pptx.chart.data.XyDataPoint", return_value=data_point_
        )


class DescribeCategoryDataPoint(object):
    def it_is_a__BaseDataPoint_object(self, series_data_):
        data_point = CategoryDataPoint(series_data_, 42, "#,##0.0")
        assert isinstance(data_point, _BaseDataPoint)
        assert data_point.number_format == "#,##0.0"
        assert data_point._series_data is series_data_

    def it_knows_its_value(self, value_fixture):
        data_point, expected_value = value_fixture
        assert data_point.value == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def value_fixture(self):
        value = 42
        data_point = CategoryDataPoint(None, value, None)
        return data_point, value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, CategorySeriesData)


class DescribeXyDataPoint(object):
    def it_is_a__BaseDataPoint_object(self, series_data_):
        data_point = XyDataPoint(series_data_, 42, 24, "00.0")
        assert isinstance(data_point, _BaseDataPoint)
        assert data_point._series_data is series_data_
        assert data_point.number_format == "00.0"

    def it_knows_its_x_y_values(self, value_fixture):
        data_point, x, y = value_fixture
        assert data_point.x == x
        assert data_point.y == y

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def value_fixture(self):
        x, y = 42, 24
        data_point = XyDataPoint(None, x, y, None)
        return data_point, x, y

    # fixture components ---------------------------------------------

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, XySeriesData)


class DescribeBubbleDataPoint(object):
    def it_is_an_XyDataPoint_subclass(self, series_data_):
        x, y, size, number_format = 1, 2, 10, "#00.0"
        data_point = BubbleDataPoint(series_data_, x, y, size, number_format)
        assert isinstance(data_point, XyDataPoint)
        assert data_point._series_data is series_data_
        assert data_point.x == x
        assert data_point.y == y
        assert data_point.number_format == number_format

    def it_knows_its_x_y_size_values(self, value_fixture):
        data_point, x, y, size = value_fixture
        assert data_point.x == x
        assert data_point.y == y
        assert data_point.bubble_size == size

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def value_fixture(self):
        x, y, size = 42, 24, 100
        data_point = BubbleDataPoint(None, x, y, size, None)
        return data_point, x, y, size

    # fixture components ---------------------------------------------

    @pytest.fixture
    def series_data_(self, request):
        return instance_mock(request, BubbleSeriesData)
